@@ -1,5 +1,5 @@
 /* GDA library
- * Copyright (C) 1998 - 2006 The GNOME Foundation.
+ * Copyright (C) 1998 - 2007 The GNOME Foundation.
  *
  * AUTHORS:
  *      Michael Lausch <michael@lausch.at>
@@ -56,7 +56,6 @@ static void gda_client_init       (GdaClient *client, GdaClientClass *klass);
 static void gda_client_finalize   (GObject *object);
 
 static void cnc_error_cb (GdaConnection *cnc, GdaConnectionEvent *error, GdaClient *client);
-static void cnc_destroyed_cb (GdaConnection *cnc, GdaClient *client);
 
 enum {
 	EVENT_NOTIFICATION,
@@ -118,18 +117,6 @@ cnc_error_cb (GdaConnection *cnc, GdaConnectionEvent *error, GdaClient *client)
 	gda_client_notify_error_event (client, cnc, error);
 }
 
-static void
-cnc_destroyed_cb (GdaConnection *cnc, GdaClient *client)
-{
-	g_assert (g_list_find (client->priv->connections, cnc));
-	g_signal_handlers_disconnect_by_func (G_OBJECT (cnc), 
-					      G_CALLBACK (cnc_destroyed_cb), client);
-	g_signal_handlers_disconnect_by_func (G_OBJECT (cnc), 
-					      G_CALLBACK (cnc_error_cb), client);
-	client->priv->connections = g_list_remove (client->priv->connections, cnc);
-	g_object_unref (cnc);
-}
-
 typedef struct {
 	GdaClient *client;
 	GdaServerProvider *provider;
@@ -212,12 +199,13 @@ static void
 gda_client_finalize (GObject *object)
 {
 	GdaClient *client = (GdaClient *) object;
+	GList *list;
 
 	g_return_if_fail (GDA_IS_CLIENT (client));
 
 	/* free memory */
-	while (client->priv->connections)
-		cnc_destroyed_cb (GDA_CONNECTION (client->priv->connections->data), client);
+	for (list = client->priv->connections; list; list = list->next)
+		g_object_unref (GDA_CONNECTION (list->data));
 
 	g_hash_table_foreach (client->priv->providers, (GHFunc) remove_weak_ref, client);
 	g_hash_table_foreach (client->priv->providers, (GHFunc) free_hash_provider, NULL);
@@ -245,7 +233,7 @@ gda_client_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaClientClass),
 			(GBaseInitFunc) NULL,
@@ -259,6 +247,7 @@ gda_client_get_type (void)
 		};
 		type = g_type_register_static (G_TYPE_OBJECT, "GdaClient", &info, 0);
 	}
+
 	return type;
 }
 
@@ -373,9 +362,6 @@ gda_client_declare_connection (GdaClient *client, GdaConnection *cnc)
 	g_object_ref (cnc);
 
 	/* signals */
-	g_signal_connect (G_OBJECT (cnc), "destroyed",
-			  G_CALLBACK (cnc_destroyed_cb), client);
-
 	g_signal_connect (G_OBJECT (cnc), "error",
 			  G_CALLBACK (cnc_error_cb), client);
 }
@@ -405,9 +391,10 @@ gda_client_declare_connection (GdaClient *client, GdaConnection *cnc)
  *  <listitem>the USERNAME= and PASSWORD= parts of the connection string in the DSN definition</listitem>
  * </itemizedlist>
  *
+ * If a new #GdaConnection is created, then the caller will hold a reference on it, and if a #GdaConnection
+ * already existing is used, then the reference count of that object will be increased by one.
  *
- * Returns: the opened connection if successful, %NULL if there is
- * an error.
+ * Returns: the opened connection if successful, %NULL if there was an error.
  */
 GdaConnection *
 gda_client_open_connection (GdaClient *client,
@@ -437,9 +424,12 @@ gda_client_open_connection (GdaClient *client,
 		cnc = gda_client_find_connection (client, dsn, username, password);
 		if (cnc &&
 		    ! (gda_connection_get_options (cnc) & GDA_CONNECTION_OPTIONS_DONT_SHARE)) {
-			g_object_ref (G_OBJECT (cnc));
-			gda_client_notify_connection_opened_event (client, cnc);
+			if (!gda_connection_open (cnc, error)) 
+				cnc = NULL;
+			else
+				g_object_ref (G_OBJECT (cnc));
 			gda_data_source_info_free (dsn_info);
+
 			return cnc;
 		}
 	}
@@ -451,8 +441,8 @@ gda_client_open_connection (GdaClient *client,
 			prv = find_or_load_provider (client, dsn_info->provider);
 	
 		if (prv) {
-			cnc = gda_connection_new (client, prv->provider, dsn, username, password, 
-						  options);
+			cnc = gda_server_provider_create_connection (client, prv->provider, dsn, username, password,
+                                                  options);
 			if (!gda_connection_open (cnc, error)) {
 				g_object_unref (cnc);
 				cnc = NULL;
@@ -472,6 +462,9 @@ gda_client_open_connection (GdaClient *client,
 	/* free memory */
 	gda_data_source_info_free (dsn_info);
 
+	if (cnc) 
+		gda_client_declare_connection (client, cnc);
+
 	return cnc;
 }
 
@@ -490,9 +483,11 @@ gda_client_open_connection (GdaClient *client,
  * a data source in the configuration. The format of @cnc_string is
  * similar to PostgreSQL and MySQL connection strings. It is a semicolumn-separated
  * series of key=value pairs. Do not add extra whitespace after the semicolumn
- * separator. The possible keys depend on the provider, but
- * these keys should work with all providers:
- * USER, PASSWORD, HOST, DATABASE, PORT
+ * separator. The possible keys depend on the provider, the "gda-sql-3.0 -L" command
+ * can be used to list the actual keys for each installed database provider.
+ *
+ * For example the connection string to open an SQLite connection to a database
+ * file named "my_data.db" in the current directory would be "DB_DIR=.;DB_NAME=my_data".
  *
  * The username and password used to actually open the connection are the first
  * non-NULL string being chosen by order from
@@ -501,7 +496,6 @@ gda_client_open_connection (GdaClient *client,
  *  <listitem>the USERNAME= and PASSWORD= parts of the @cnc_string</listitem>
  * </itemizedlist>
  *
-
  *
  * Returns: the opened connection if successful, %NULL if there is
  * an error.
@@ -543,13 +537,8 @@ gda_client_open_connection_from_string (GdaClient *client,
 			prv = find_or_load_provider (client, provider_id);
 	
 		if (prv) {
-			cnc = g_object_new (GDA_TYPE_CONNECTION, 
-					    "client", client, 
-					    "provider_obj", prv->provider, 
-					    "cnc_string", cnc_string, 
-					    "username", username, 
-					    "password", password, 
-					    "options", options, NULL);
+			cnc = gda_server_provider_create_connection_from_string (client, prv->provider,
+										 cnc_string, username, password, options);
 			if (!gda_connection_open (cnc, error)) {
 				g_object_unref (cnc);
 				cnc = NULL;
@@ -626,9 +615,9 @@ gda_client_find_connection (GdaClient *client,
 		const gchar *tmp_dsn, *tmp_usr, *tmp_pwd;
 
 		cnc = GDA_CONNECTION (l->data);
-		tmp_dsn = dsn ? dsn : gda_connection_get_dsn (cnc);
-		tmp_usr = username ? username : gda_connection_get_username (cnc);
-		tmp_pwd = password ? password : gda_connection_get_password (cnc);
+		tmp_dsn = gda_connection_get_dsn (cnc);
+		tmp_usr = gda_connection_get_username (cnc);
+		tmp_pwd = gda_connection_get_password (cnc);
 
 		if (!strcmp (tmp_dsn ? tmp_dsn : "", dsn_info->name ? dsn_info->name : "")
 		    && !strcmp (tmp_usr ? tmp_usr : "", username ? username : "")
@@ -676,12 +665,12 @@ gda_client_notify_event (GdaClient *client,
 			 GdaClientEvent event,
 			 GdaParameterList *params)
 {
-	g_return_if_fail (GDA_IS_CLIENT (client));
+	if (!client)
+		return;
 
-	if (!cnc || g_list_find (client->priv->connections, cnc)) {
-		g_signal_emit (G_OBJECT (client), gda_client_signals[EVENT_NOTIFICATION], 0,
-			       cnc, event, params);
-	}
+	g_return_if_fail (GDA_IS_CLIENT (client));
+	g_signal_emit (G_OBJECT (client), gda_client_signals[EVENT_NOTIFICATION], 0,
+		       cnc, event, params);
 }
 
 /**
@@ -699,6 +688,9 @@ gda_client_notify_error_event (GdaClient *client, GdaConnection *cnc, GdaConnect
 	GdaParameter *param;
 	GValue *value;
 
+	if (!client)
+		return;
+
 	g_return_if_fail (GDA_IS_CLIENT (client));
 	g_return_if_fail (error != NULL);
 
@@ -727,6 +719,9 @@ gda_client_notify_error_event (GdaClient *client, GdaConnection *cnc, GdaConnect
 void
 gda_client_notify_connection_opened_event (GdaClient *client, GdaConnection *cnc)
 {
+	if (!client)
+		return;
+
 	g_return_if_fail (GDA_IS_CLIENT (client));
 	g_return_if_fail (GDA_IS_CONNECTION (cnc));
 
@@ -744,6 +739,9 @@ gda_client_notify_connection_opened_event (GdaClient *client, GdaConnection *cnc
 void
 gda_client_notify_connection_closed_event (GdaClient *client, GdaConnection *cnc)
 {
+	if (!client)
+		return;
+
 	g_return_if_fail (GDA_IS_CLIENT (client));
 	g_return_if_fail (GDA_IS_CONNECTION (cnc));
 
