@@ -46,7 +46,7 @@
 struct _GdaConnectionPrivate {
 	GdaClient            *client;
 	GdaServerProvider    *provider_obj;
-	GdaConnectionOptions options; /* ORed flags */
+	GdaConnectionOptions  options; /* ORed flags */
 	gchar                *dsn;
 	gchar                *cnc_string;
 	gchar                *provider;
@@ -57,6 +57,7 @@ struct _GdaConnectionPrivate {
 	GList                *recset_list;
 
 	GdaTransactionStatus *trans_status;
+	GHashTable           *prepared_stmts;
 };
 
 static void gda_connection_class_init (GdaConnectionClass *klass);
@@ -223,6 +224,8 @@ gda_connection_dispose (GObject *object)
 	/* free memory */
 	gda_connection_close_no_warning (cnc);
 
+	gda_connection_destroy_prepared_statement_hash (cnc);
+
 	if (cnc->priv->provider_obj) {
 		g_object_unref (G_OBJECT (cnc->priv->provider_obj));
 		cnc->priv->provider_obj = NULL;
@@ -239,11 +242,6 @@ gda_connection_dispose (GObject *object)
 		cnc->priv->trans_status = NULL;
 	}
 
-        if (cnc->priv->client) {
-		g_object_unref (cnc->priv->client);
-		cnc->priv->client = NULL;
-	}
-
 	/* chain to parent class */
 	parent_class->dispose (object);
 }
@@ -287,9 +285,9 @@ GQuark gda_connection_error_quark (void)
 GType
 gda_connection_get_type (void)
 {
-   static GType type = 0;
+	static GType type = 0;
 
-	if (type == 0) {
+	if (G_UNLIKELY (type == 0)) {
 		static GTypeInfo info = {
 			sizeof (GdaConnectionClass),
 			(GBaseInitFunc) NULL,
@@ -300,8 +298,9 @@ gda_connection_get_type (void)
 			0,
 			(GInstanceInitFunc) gda_connection_init
 		};
-	type = g_type_register_static (PARENT_TYPE, "GdaConnection", &info, 0);
+		type = g_type_register_static (PARENT_TYPE, "GdaConnection", &info, 0);
 	}
+
 	return type;
 }
 
@@ -317,12 +316,12 @@ gda_connection_set_property (GObject *object,
         if (cnc->priv) {
                 switch (param_id) {
                 case PROP_CLIENT:
-                        if(cnc->priv->client)
-                          g_object_unref(cnc->priv->client);
+                        if (cnc->priv->client)
+				g_object_unref(cnc->priv->client);
 
 			cnc->priv->client = g_value_get_object (value);
-                        g_object_ref(cnc->priv->client);
-
+			if (cnc->priv->client)
+				g_object_ref (cnc->priv->client);
 			break;
                 case PROP_DSN:
 			gda_connection_set_dsn (cnc, g_value_get_string (value));
@@ -335,7 +334,7 @@ gda_connection_set_property (GObject *object,
                         break;
                 case PROP_PROVIDER_OBJ:
                         if (cnc->priv->provider_obj)
-                          g_object_unref(cnc->priv->provider_obj);
+				g_object_unref(cnc->priv->provider_obj);
 
 			cnc->priv->provider_obj = g_value_get_object (value);
 			g_object_ref (G_OBJECT (cnc->priv->provider_obj));
@@ -554,6 +553,16 @@ gda_connection_open (GdaConnection *cnc, GError **error)
 	g_free (real_username);
 	g_free (real_password);
 
+	if (cnc->priv->is_open) {
+#ifdef GDA_DEBUG_signal
+        g_print (">> 'CONN_OPENED' from %s\n", __FUNCTION__);
+#endif
+        g_signal_emit (G_OBJECT (cnc), gda_connection_signals[CONN_OPENED], 0);
+#ifdef GDA_DEBUG_signal
+        g_print ("<< 'CONN_OPENED' from %s\n", __FUNCTION__);
+#endif
+	}
+
 	return cnc->priv->is_open;
 }
 
@@ -970,10 +979,14 @@ gda_connection_add_event (GdaConnection *cnc, GdaConnectionEvent *event)
 	case GDA_CONNECTION_EVENT_ERROR:
 		str = "ERROR";
 		break;
+	case GDA_CONNECTION_EVENT_COMMAND:
+		str = "COMMAND";
+                break;
 	default:
 		break;
 	}
-	g_print ("EVENT> %s: %s (on cnx %p)\n", str, gda_connection_event_get_description (event), cnc);
+	g_print ("EVENT> %s: %s (on cnx %p, %s)\n", str, gda_connection_event_get_description (event), cnc,
+		gda_connection_event_get_sqlstate (event));
 #endif
 	if (gda_connection_event_get_event_type (event) == GDA_CONNECTION_EVENT_ERROR)
 		g_signal_emit (G_OBJECT (cnc), gda_connection_signals[ERROR], 0, event);
@@ -981,7 +994,7 @@ gda_connection_add_event (GdaConnection *cnc, GdaConnectionEvent *event)
 
 /**
  * gda_connection_add_event_string
- * @cnc: a #GdaServerConnection object.
+ * @cnc: a #GdaConnection object.
  * @str: a format string (see the printf(3) documentation).
  * @...: the arguments to insert in the error message.
  *
@@ -1125,6 +1138,11 @@ gda_connection_change_database (GdaConnection *cnc, const gchar *name)
  * <itemizedlist>
  *   <listitem><para>a "ITER_MODEL_ONLY" parameter of type #G_TYPE_BOOLEAN which, if set to TRUE
  *             will preferably return a data model which can be accessed only using an iterator.</para></listitem>
+ *   <listitem><para>a "ITER_CHUNCK_SIZE" parameter of type #G_TYPE_INT which specifies, if "ITER_MODEL_ONLY"
+ *             is set to TRUE, how many rows are fetched (and cached) from the database everytime the iterator needs
+ *             to access a row for which the data must be fetched from the database. For the providers which support this
+ *             setting this parameter to a value greater than one will increase mamory usage but reduce the time spent
+ *             to transfer data from the database.</para></listitem>
  * </itemizedlist>
  *
  * Returns: a list of #GdaDataModel and #GdaParameterList or %NULL, as returned by the underlying
@@ -1142,9 +1160,6 @@ gda_connection_execute_command (GdaConnection *cnc, GdaCommand *cmd,
 	g_return_val_if_fail (cmd != NULL, NULL);
 	g_return_val_if_fail (cnc->priv->provider_obj, NULL);
 
-	/* clean any previous connection events */
-	gda_connection_clear_events_list (cnc);
-
 	/* execute the command on the provider */
 	retval = gda_server_provider_execute_command (cnc->priv->provider_obj,
 						      cnc, cmd, params);
@@ -1253,7 +1268,7 @@ gda_connection_execute_select_command (GdaConnection *cnc, GdaCommand *cmd,
 		else if (nb >= 0)
 			str = g_strdup_printf (_("(%d row)"), nb);
 		else
-			str = g_strdup_printf (_("(unknown number of rows)"), nb);
+			str = g_strdup_printf (_("(unknown number of rows)"));
 			
 		gda_connection_event_set_description (event, str);
 		g_free (str);
@@ -1549,7 +1564,7 @@ gda_connection_supports_feature (GdaConnection *cnc, GdaConnectionFeature featur
  * <link linkend="libgda-provider-get-schema">get_schema() virtual method for providers</link> for more details.
  *
  * Returns: a #GdaDataModel containing the data required. The caller is responsible
- * for freeing the returned model using g_object_unref().
+ * of freeing the returned model using g_object_unref().
  */
 GdaDataModel *
 gda_connection_get_schema (GdaConnection *cnc,
@@ -1861,3 +1876,125 @@ gda_connection_internal_change_transaction_state (GdaConnection *cnc,
 	g_print ("<< 'TRANSACTION_STATUS_CHANGED' from %s\n", __FUNCTION__);
 #endif
 }
+
+void
+gda_connection_force_status (GdaConnection *cnc, gboolean opened)
+{
+	g_return_if_fail (GDA_IS_CONNECTION (cnc));
+
+	if (opened && !cnc->priv->is_open) {
+		cnc->priv->is_open = TRUE;
+#ifdef GDA_DEBUG_signal
+		g_print (">> 'CONN_OPENED' from %s\n", __FUNCTION__);
+#endif
+		g_signal_emit (G_OBJECT (cnc), gda_connection_signals[CONN_OPENED], 0);
+#ifdef GDA_DEBUG_signal
+		g_print ("<< 'CONN_OPENED' from %s\n", __FUNCTION__);
+#endif
+		if (cnc->priv->client)
+			gda_client_notify_connection_opened_event (cnc->priv->client, cnc);
+		return;
+	}
+
+	if (!opened && cnc->priv->is_open) {
+		cnc->priv->is_open = FALSE;
+
+#ifdef GDA_DEBUG_signal
+		g_print (">> 'CONN_TO_CLOSE' from %s\n", __FUNCTION__);
+#endif
+		g_signal_emit (G_OBJECT (cnc), gda_connection_signals[CONN_TO_CLOSE], 0);
+#ifdef GDA_DEBUG_signal
+		g_print ("<< 'CONN_TO_CLOSE' from %s\n", __FUNCTION__);
+#endif
+#ifdef GDA_DEBUG_signal
+		g_print (">> 'CONN_CLOSED' from %s\n", __FUNCTION__);
+#endif
+		g_signal_emit (G_OBJECT (cnc), gda_connection_signals[CONN_CLOSED], 0);
+#ifdef GDA_DEBUG_signal
+		g_print ("<< 'CONN_CLOSED' from %s\n", __FUNCTION__);
+#endif
+
+		if (cnc->priv->client)
+			gda_client_notify_connection_closed_event (cnc->priv->client, cnc);
+	}
+}
+
+/*
+ * Prepared statements handling
+ */
+
+static void prepared_stms_query_destroyed_cb (GdaQuery *query, GdaConnection *cnc);
+static void
+prepared_stms_foreach_func (GdaQuery *query, gpointer prepared_stmt, GdaConnection *cnc)
+{
+	g_signal_handlers_disconnect_by_func (query, G_CALLBACK (prepared_stms_query_destroyed_cb), cnc);
+}
+
+void 
+gda_connection_destroy_prepared_statement_hash (GdaConnection *cnc)
+{
+	if (!cnc->priv->prepared_stmts)
+		return;
+
+	g_hash_table_foreach (cnc->priv->prepared_stmts, (GHFunc) prepared_stms_foreach_func, cnc);
+	g_hash_table_destroy (cnc->priv->prepared_stmts);
+	cnc->priv->prepared_stmts = NULL;
+}
+
+void
+gda_connection_init_prepared_statement_hash (GdaConnection *cnc, GDestroyNotify stmt_destroy_func)
+{
+	g_return_if_fail (GDA_IS_CONNECTION (cnc));
+	g_return_if_fail (cnc->priv);
+
+	if (!cnc->priv->prepared_stmts)
+		cnc->priv->prepared_stmts = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, stmt_destroy_func);
+}
+
+static void 
+prepared_stms_query_destroyed_cb (GdaQuery *query, GdaConnection *cnc)
+{
+	g_signal_handlers_disconnect_by_func (query, G_CALLBACK (prepared_stms_query_destroyed_cb), cnc);
+	g_hash_table_remove (cnc->priv->prepared_stmts, query);
+}
+
+void 
+gda_connection_add_prepared_statement (GdaConnection *cnc, GdaQuery *query, gpointer prepared_stmt)
+{
+	g_return_if_fail (GDA_IS_CONNECTION (cnc));
+	g_return_if_fail (cnc->priv);
+
+	if (!cnc->priv->prepared_stmts) {
+		g_warning (_("Prepared statements hash not initialized, "
+			     "call gda_connection_init_prepared_statement_hash() first"));
+		return;
+	}
+	g_hash_table_remove (cnc->priv->prepared_stmts, query);
+	g_hash_table_insert (cnc->priv->prepared_stmts, query, prepared_stmt);
+	
+	/* destroy the prepared statement if query is destroyed, or changes */
+	gda_object_connect_destroy (GDA_OBJECT (query), G_CALLBACK (prepared_stms_query_destroyed_cb), cnc);
+	g_signal_connect (G_OBJECT (query), "changed", G_CALLBACK (prepared_stms_query_destroyed_cb), cnc);
+
+}
+
+gpointer
+gda_connection_get_prepared_statement (GdaConnection *cnc, GdaQuery *query)
+{
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+	g_return_val_if_fail (cnc->priv, NULL);
+
+	if (!cnc->priv->prepared_stmts) {
+		g_warning (_("Prepared statements hash not initialized, "
+			     "call gda_connection_init_prepared_statement_hash() first"));
+		return NULL;
+	}
+	return g_hash_table_lookup (cnc->priv->prepared_stmts, query);
+}
+
+void
+gda_connection_del_prepared_statement (GdaConnection *cnc, GdaQuery *query)
+{
+	g_return_if_fail (GDA_IS_CONNECTION (cnc));
+	prepared_stms_query_destroyed_cb (query, cnc);
+}
