@@ -390,7 +390,7 @@ gda_data_model_hash_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaDataModelHashClass),
 			(GBaseInitFunc) NULL,
@@ -406,6 +406,7 @@ gda_data_model_hash_get_type (void)
 					       "GdaDataModelHash",
 					       &info, 0);
 	}
+
 	return type;
 }
 
@@ -427,7 +428,7 @@ gda_data_model_hash_new (gint cols)
  * gda_data_model_hash_insert_row
  * @model: the #GdaDataModelHash which is gonna hold the row.
  * @rownum: the number of the row.
- * @row: the row to insert. The model is responsible for freeing it with g_object_unref().
+ * @row: the row to insert. The model is responsible of freeing it!
  *
  * Inserts a @row in the @model.
  */
