@@ -108,7 +108,7 @@ gda_data_model_iter_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaDataModelIterClass),
 			(GBaseInitFunc) NULL,
@@ -459,9 +459,9 @@ gda_data_model_iter_set_property (GObject *object,
 					gda_parameter_set_exists_default_value (param, TRUE);
 				gda_parameter_list_add_param ((GdaParameterList *) iter, param);
 				g_object_set_data (G_OBJECT (param), "model_col", GINT_TO_POINTER (col + 1));
-				g_object_unref (param);
-				gda_object_connect_destroy (ptr,
+				gda_object_connect_destroy (param,
 							    G_CALLBACK (destroyed_param_cb), iter);
+				g_object_unref (param);
 			}
 		}
 		case PROP_FORCED_MODEL: {
@@ -544,7 +544,7 @@ gda_data_model_iter_can_be_moved (GdaDataModelIter *iter)
 	if (!gda_data_model_iter_is_valid (iter))
 		return TRUE;
 
-	/* optionally validate the row change */
+	/* optionaly validate the row change */
 	g_signal_emit (G_OBJECT (iter),
 		       gda_data_model_iter_signals[ROW_TO_CHANGE],
 		       0, iter->priv->row, &move_ok);
