@@ -39,6 +39,7 @@
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
+#include "csv.h"
 
 #define PARENT_TYPE G_TYPE_OBJECT
 #define CLASS(model) (GDA_DATA_MODEL_CLASS (G_OBJECT_GET_CLASS (model)))
@@ -61,7 +62,7 @@ gda_data_model_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaDataModelClass),
 			(GBaseInitFunc) gda_data_model_class_init,
@@ -355,7 +356,7 @@ gda_data_model_get_n_columns (GdaDataModel *model)
 	if (GDA_DATA_MODEL_GET_CLASS (model)->i_get_n_columns)
 		return (GDA_DATA_MODEL_GET_CLASS (model)->i_get_n_columns) (model);
 	else {
-		g_warning ("%s() method not supported\n", __FUNCTION__);
+		/* method not supported */
 		return -1;
 	}
 }
@@ -383,7 +384,7 @@ gda_data_model_describe_column (GdaDataModel *model, gint col)
 	if (GDA_DATA_MODEL_GET_CLASS (model)->i_describe_column)
 		return (GDA_DATA_MODEL_GET_CLASS (model)->i_describe_column) (model, col);
 	else {
-		g_warning ("%s() method not supported\n", __FUNCTION__);
+		/* method not supported */
 		return NULL;
 	}
 }
@@ -418,6 +419,57 @@ gda_data_model_get_column_index_by_name (GdaDataModel *model, const gchar *name)
 	return ret;
 }
 
+/**
+ * gda_data_model_get_column_name
+ * @model: a #GdaDataModel object.
+ * @col: column number.
+ *
+ * Since: 3.2
+ *
+ * Returns: the name for the given column in a data model object.
+ */
+const gchar *
+gda_data_model_get_column_name (GdaDataModel *model, gint col)
+{
+	GdaColumn *column;
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
+
+	column = gda_data_model_describe_column (model, col);
+	if (column)
+		return gda_column_get_name (column);
+	else {
+		g_warning ("%s(): can't get GdaColumn object for column %d\n", __FUNCTION__, col);
+		return NULL;
+	}
+}
+
+/**
+ * gda_data_model_set_column_name
+ * @model: a #GdaDataModel object.
+ * @col: column number
+ * @name: name for the given column.
+ *
+ * Sets the @name of the given @col in @model, and if its title is not set, also sets the
+ * title to @name.
+ *
+ * Since: 3.2
+ */
+void
+gda_data_model_set_column_name (GdaDataModel *model, gint col, const gchar *name)
+{
+	GdaColumn *column;
+	g_return_if_fail (GDA_IS_DATA_MODEL (model));
+
+	column = gda_data_model_describe_column (model, col);
+	if (column) {
+		gda_column_set_name (column, name);
+		if (!gda_column_get_title (column))
+			gda_column_set_title (column, name);
+	}
+	else 
+		g_warning ("%s(): can't get GdaColumn object for column %d\n", __FUNCTION__, col);
+}
+
 
 /**
  * gda_data_model_get_column_title
@@ -436,7 +488,7 @@ gda_data_model_get_column_title (GdaDataModel *model, gint col)
 	if (column)
 		return gda_column_get_title (column);
 	else {
-		g_warning ("%s() method not supported\n", __FUNCTION__);
+		g_warning ("%s(): can't get GdaColumn object for column %d\n", __FUNCTION__, col);
 		return NULL;
 	}
 }
@@ -459,7 +511,7 @@ gda_data_model_set_column_title (GdaDataModel *model, gint col, const gchar *tit
 	if (column)
 		gda_column_set_title (column, title);
 	else 
-		g_warning ("%s() method not supported\n", __FUNCTION__);
+		g_warning ("%s(): can't get GdaColumn object for column %d\n", __FUNCTION__, col);
 }
 
 /**
@@ -488,7 +540,7 @@ gda_data_model_get_value_at (GdaDataModel *model, gint col, gint row)
 	if (GDA_DATA_MODEL_GET_CLASS (model)->i_get_value_at)
 		return (GDA_DATA_MODEL_GET_CLASS (model)->i_get_value_at) (model, col, row);
 	else {
-		g_warning ("%s() method not supported\n", __FUNCTION__);
+		/* method not supported */
 		return NULL;
 	}
 }
@@ -528,7 +580,7 @@ gda_data_model_get_value_at_col_name (GdaDataModel *model,
  * @col: a valid column number
  * @row: a valid row number, or -1
  *
- * Get the attributes of the value stored at (row, col) in @proxy, which
+ * Get the attributes of the value stored at (row, col) in @model, which
  * is an ORed value of #GdaValueAttribute flags. As a special case, if
  * @row is -1, then the attributes returned correspond to a "would be" value
  * if a row was added to @model.
@@ -564,7 +616,7 @@ gda_data_model_set_value_at (GdaDataModel *model, gint col, gint row, const GVal
 	if (GDA_DATA_MODEL_GET_CLASS (model)->i_set_value_at)
 		return (GDA_DATA_MODEL_GET_CLASS (model)->i_set_value_at) (model, col, row, value, error);
 	else {
-		g_warning ("%s() method not supported\n", __FUNCTION__);
+		/* method not supported */
 		return FALSE;
 	}
 }
@@ -589,7 +641,7 @@ gda_data_model_set_values (GdaDataModel *model, gint row, GList *values, GError
 	if (GDA_DATA_MODEL_GET_CLASS (model)->i_set_values)
 		return (GDA_DATA_MODEL_GET_CLASS (model)->i_set_values) (model, row, values, error);
 	else {
-		g_warning ("%s() method not supported\n", __FUNCTION__);
+		/* method not supported */
 		return FALSE;
 	}
 }
@@ -867,7 +919,7 @@ gda_data_model_append_values (GdaDataModel *model, const GList *values, GError *
 	if (GDA_DATA_MODEL_GET_CLASS (model)->i_append_values) 
 		return (GDA_DATA_MODEL_GET_CLASS (model)->i_append_values) (model, values, error);
 	else {
-		g_warning ("%s() method not supported\n", __FUNCTION__);
+		/* method not supported */
 		return -1;
 	}
 }
@@ -886,10 +938,16 @@ gda_data_model_append_row (GdaDataModel *model, GError **error)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
 
+	if (! (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_INSERT)) {
+		g_set_error (error, 0, 0,
+			     _("Model does not allow row insertion"));
+		return FALSE;
+	}
+
 	if (GDA_DATA_MODEL_GET_CLASS (model)->i_append_row) 
 		return (GDA_DATA_MODEL_GET_CLASS (model)->i_append_row) (model, error);
 	else {
-		g_warning ("%s() method not supported\n", __FUNCTION__);
+		/* method not supported */
 		return -1;
 	}
 }
@@ -910,10 +968,16 @@ gda_data_model_remove_row (GdaDataModel *model, gint row, GError **error)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
 
+	if (! (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_DELETE)) {
+		g_set_error (error, 0, 0,
+			     _("Model does not allow row deletion"));
+		return FALSE;
+	}
+
 	if (GDA_DATA_MODEL_GET_CLASS (model)->i_remove_row)
 		return (GDA_DATA_MODEL_GET_CLASS (model)->i_remove_row) (model, row, error);
 	else {
-		g_warning ("%s() method not supported\n", __FUNCTION__);
+		/* method not supported */
 		return FALSE;
 	}
 }
@@ -1008,7 +1072,7 @@ gda_data_model_send_hint (GdaDataModel *model, GdaDataModelHint hint, const GVal
 }
 
 static gchar *export_to_text_separated (GdaDataModel *model, const gint *cols, gint nb_cols, 
-					const gint *rows, gint nb_rows, gchar sep);
+					const gint *rows, gint nb_rows, gchar sep, gchar quote, gboolean field_quotes);
 
 
 /**
@@ -1021,14 +1085,9 @@ static gchar *export_to_text_separated (GdaDataModel *model, const gint *cols, g
  * @nb_rows: the number of rows in @rows
  * @options: list of options for the export
  *
- * Exports data contained in @model to a string; the format is specified using the @format argument.
- *
- * Specifically, the parameters in the @options list can be:
- * <itemizedlist>
- *   <listitem><para>"SEPARATOR": a string value of which the first character is used as a separator in case of CSV export
- *             </para></listitem>
- *   <listitem><para>"NAME": a string value used to name the exported data if the export format is XML</para></listitem>
- * </itemizedlist>
+ * Exports data contained in @model to a string; the format is specified using the @format argument, see the
+ * gda_data_model_export_to_file() documentation for more information about the @options argument (except for the
+ * "OVERWRITE" option).
  *
  * Returns: a new string.
  */
@@ -1080,6 +1139,9 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
 
 	case GDA_DATA_MODEL_IO_TEXT_SEPARATED: {
 		gchar sep = ',';
+		gchar quote = '"';
+		gboolean field_quote = TRUE;
+
 		if (options) {
 			GdaParameter *param;
 
@@ -1097,10 +1159,33 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
 				else
 					g_warning (_("The 'SEPARATOR' parameter must hold a string value, ignored."));
 			}
+			param = gda_parameter_list_find_param (options, "QUOTE");
+			if (param) {
+				const GValue *value;
+				value = gda_parameter_get_value (param);
+				if (value && gda_value_isa ((GValue *) value, G_TYPE_STRING)) {
+					const gchar *str;
+
+					str = g_value_get_string ((GValue *) value);
+					if (str && *str)
+						quote = *str;
+				}
+				else 
+					g_warning (_("The 'QUOTE' parameter must hold a string value, ignored."));
+			}
+			param = gda_parameter_list_find_param (options, "FIELD_QUOTE");
+			if (param) {
+				const GValue *value;
+				value = gda_parameter_get_value (param);
+				if (value && gda_value_isa ((GValue *) value, G_TYPE_BOOLEAN)) 
+					field_quote = g_value_get_boolean ((GValue *) value);
+				else 
+					g_warning (_("The 'FIELD_QUOTE' parameter must hold a boolean value, ignored."));
+			}
 		}
 
 		if (cols)
-			return export_to_text_separated (model, cols, nb_cols, rows, nb_rows, sep);
+			return export_to_text_separated (model, cols, nb_cols, rows, nb_rows, sep, quote, field_quote);
 		else {
 			gchar *retval;
 			gint *rcols, rnb_cols, i;
@@ -1111,7 +1196,7 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
 			rcols = g_new (gint, rnb_cols);
 			for (i = 0; i < rnb_cols; i++)
 				rcols[i] = i;
-			retval = export_to_text_separated (model, rcols, rnb_cols, rows, nb_rows, sep);
+			retval = export_to_text_separated (model, rcols, rnb_cols, rows, nb_rows, sep, quote, field_quote);
 			g_free (rcols);
 			
 			return retval;
@@ -1142,6 +1227,10 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
  * <itemizedlist>
  *   <listitem><para>"SEPARATOR": a string value of which the first character is used as a separator in case of CSV export
  *             </para></listitem>
+ *   <listitem><para>"QUOTE": a string value of which the first character is used as a quote character in case of CSV export
+ *             </para></listitem>
+ *   <listitem><para>"FIELD_QUOTE": a boolean value which can be set to FALSE if no quote around the individual fields 
+ *             is requeted, in case of CSV export</para></listitem>
  *   <listitem><para>"NAME": a string value used to name the exported data if the export format is XML</para></listitem>
  *   <listitem><para>"OVERWRITE": a boolean value which tells if the file must be over-written if it already exists.
  *             </para></listitem>
@@ -1198,7 +1287,8 @@ gda_data_model_export_to_file (GdaDataModel *model, GdaDataModelIOFormat format,
 }
 
 static gchar *
-export_to_text_separated (GdaDataModel *model, const gint *cols, gint nb_cols, const gint *rows, gint nb_rows, gchar sep)
+export_to_text_separated (GdaDataModel *model, const gint *cols, gint nb_cols, const gint *rows, gint nb_rows, 
+			  gchar sep, gchar quote, gboolean field_quotes)
 {
 	GString *str;
 	gchar *retval;
@@ -1223,14 +1313,26 @@ export_to_text_separated (GdaDataModel *model, const gint *cols, gint nb_cols, c
 			value = (GValue *) gda_data_model_get_value_at (model, cols[c], rows ? rows[r] : r);
 			if (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN)
 				txt = g_strdup (g_value_get_boolean (value) ? "TRUE" : "FALSE");
-			else
-				txt = gda_value_stringify (value);
+			else {
+				gchar *tmp;
+				gsize len, size;
+				
+				tmp = gda_value_stringify (value);
+				len = strlen (tmp);
+				size = 2 * len + 3;
+				txt = g_new (gchar, size);
+
+				len = csv_write2 (txt, size, tmp, len, quote);
+				txt [len] = 0;
+				if (!field_quotes) {
+					txt [len - 1] = 0;
+					memmove (txt, txt+1, len);
+				}
+			}
 			if (c > 0)
 				str = g_string_append_c (str, sep);
-			str = g_string_append_c (str, '"');
-			str = g_string_append (str, txt);
-			str = g_string_append_c (str, '"');
 
+			str = g_string_append (str, txt);
 			g_free (txt);
 		}
 	}
@@ -1374,15 +1476,13 @@ find_column_from_id (GdaDataModel *model, const gchar *colid, gint *pos)
 	/* assume @colid is the ID of a column */
 	nbcols = gda_data_model_get_n_columns (model);
 	for (c = 0; !column && (c < nbcols); c++) {
-		gchar *id;
+		const gchar *id;
 		column = gda_data_model_describe_column (model, c);
 		g_object_get (column, "id", &id, NULL);
 		if (!id || strcmp (id, colid)) 
 			column = NULL;
 		else
 			*pos = c;
-		if(id != NULL)
-			g_free(id);
 	}
 
 	/* if no column has been found, assumr @colid is like "_%d" where %d is a column number */
@@ -1472,16 +1572,13 @@ add_xml_row (GdaDataModel *model, xmlNodePtr xml_row, GError **error)
 
 		if (!isnull) {
 			value = g_new0 (GValue, 1);
-			gchar* nodeval = (gchar*)xmlNodeGetContent (xml_field);
-			if (!gda_value_set_from_string (value, nodeval, gdatype)) {
+			if (!gda_value_set_from_string (value, (gchar*)xmlNodeGetContent (xml_field), gdatype)) {
 				g_free (value);
-				xmlFree(nodeval);
 				g_set_error (error, 0, 0, _("Cannot interpret string as a valid %s value"), 
 					     gda_g_type_to_string (gdatype));
 				retval = FALSE;
 				break;
 			}
-			xmlFree(nodeval);
 		}
 		else
 			g_free (isnull);
@@ -1521,9 +1618,9 @@ add_xml_row (GdaDataModel *model, xmlNodePtr xml_row, GError **error)
 /**
  * gda_data_model_add_data_from_xml_node
  * @model: a #GdaDataModel.
- * @node: an XML node representing a &lt;gda_array_data&gt; XML node.
+ * @node: a XML node representing a &lt;gda_array_data&gt; XML node.
  *
- * Adds the data from an XML node to the given data model (see the DTD for that node
+ * Adds the data from a XML node to the given data model (see the DTD for that node
  * in the $prefix/share/libgda/dtd/libgda-array.dtd file).
  *
  * Returns: %TRUE if successful, %FALSE otherwise.
@@ -1863,11 +1960,11 @@ gda_data_model_dump (GdaDataModel *model, FILE *to_stream)
 	str = gda_data_model_dump_as_string (model);
 	g_fprintf (to_stream, "%s", str);
 	g_free (str);
-#ifdef GDA_DEBUG_model_attributes
-	str= real_gda_data_model_dump_as_string (model, TRUE);
-	g_fprintf (to_stream, "%s", str);
-	g_free (str);
-#endif
+	if (getenv ("GDA_DATA_MODEL_DUMP_ATTRIBUTES")) {
+		str= real_gda_data_model_dump_as_string (model, TRUE);
+		g_fprintf (to_stream, "%s", str);
+		g_free (str);
+	}
 }
 
 /**
@@ -1886,15 +1983,42 @@ gda_data_model_dump_as_string (GdaDataModel *model)
 	return real_gda_data_model_dump_as_string (model, FALSE);
 }
 
+static void
+string_get_dimensions (const gchar *string, gint *width, gint *rows)
+{
+	const gchar *ptr;
+	gint w = 0, h = 0, maxw = 0;
+	for (ptr = string; *ptr; ptr = g_utf8_next_char (ptr)) {
+		if (*ptr == '\n') {
+			h++;
+			maxw = MAX (maxw, w);
+			w = 0;
+		}
+		else
+			w++;
+	}
+	maxw = MAX (maxw, w);
+
+	if (width)
+		*width = maxw;
+	if (rows)
+		*rows = h;
+}
+
 static gchar *
 real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attributes)
 {
+#define MULTI_LINE_NO_SEPARATOR
+
 	GString *string;
 	gchar *offstr, *str;
 	gint n_cols, n_rows;
 	gint *cols_size;
 	gboolean *cols_is_num;
 	gchar *sep_col  = " | ";
+#ifdef MULTI_LINE_NO_SEPARATOR
+	gchar *sep_col_e  = "   ";
+#endif
 	gchar *sep_row  = "-+-";
 	gchar sep_fill = '-';
 	gint i, j;
@@ -1918,7 +2042,7 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 		GdaColumn *gdacol;
 		GType coltype;
 
-#ifdef GDA_DEBUG
+#ifdef GDA_DEBUG_NO
 		{
 			GdaColumn *col = gda_data_model_describe_column (model, i);
 			g_print ("Model col %d has type %s\n", i, gda_g_type_to_string (gda_column_get_g_type (col)));
@@ -1960,15 +2084,19 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 				value = gda_data_model_get_value_at (model, i, j);
 				str = value ? gda_value_stringify ((GValue*)value) : g_strdup ("_null_");
 				if (str) {
-					cols_size [i] = MAX (cols_size [i], g_utf8_strlen (str, -1));
+					gint w;
+					string_get_dimensions (str, &w, NULL);
+					cols_size [i] = MAX (cols_size [i], w);
 					g_free (str);
 				}
 			}
 			else {
 				GdaValueAttribute attrs;
+				gint w;
 				attrs = gda_data_model_get_attributes_at (model, i, j);
 				str = g_strdup_printf ("%u", attrs);
-				cols_size [i] = MAX (cols_size [i], g_utf8_strlen (str, -1));
+				string_get_dimensions (str, &w, NULL);
+				cols_size [i] = MAX (cols_size [i], w);
 				g_free (str);
 			}
 		}
@@ -2006,6 +2134,11 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 	/* ... and data */
 	if (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_RANDOM) {
 		for (j = 0; j < n_rows; j++) {
+			/* determine height for each column in that row */
+			gint *cols_height = g_new (gint, n_cols);
+			gchar ***cols_str = g_new (gchar **, n_cols);
+			gint k, kmax = 0;
+
 			for (i = 0; i < n_cols; i++) {
 				if (!dump_attributes) {
 					value = gda_data_model_get_value_at (model, i, j);
@@ -2016,24 +2149,84 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 					attrs = gda_data_model_get_attributes_at (model, i, j);
 					str = g_strdup_printf ("%u", attrs);
 				}
-				if (i != 0)
-					g_string_append_printf (string, "%s", sep_col);
-				if (cols_is_num [i])
-					g_string_append_printf (string, "%*s", cols_size [i], str);
+				if (str) {
+					cols_str [i] = g_strsplit (str, "\n", -1);
+					g_free (str);
+					cols_height [i] = g_strv_length (cols_str [i]);
+					kmax = MAX (kmax, cols_height [i]);
+				}
 				else {
-					gint j, max;
-					if (str) {
-						g_string_append_printf (string, "%s", str);
-						max = cols_size [i] - g_utf8_strlen (str, -1);
-					}
-					else
-						max = cols_size [i];
-					for (j = 0; j < max; j++)
-						g_string_append_c (string, ' ');
+					cols_str [i] = NULL;
+					cols_height [i] = 0;
 				}
-				g_free (str);
 			}
-			g_string_append_c (string, '\n');
+
+			for (k = 0; k < kmax; k++) {
+				for (i = 0; i < n_cols; i++) {
+					gboolean align_center = FALSE;
+					if (i != 0) {
+#ifdef MULTI_LINE_NO_SEPARATOR
+						if (k != 0)
+							g_string_append_printf (string, "%s", sep_col_e);
+						else
+#endif
+							g_string_append_printf (string, "%s", sep_col);
+					}
+					if (k < cols_height [i]) 
+						str = (cols_str[i])[k];
+					else {
+#ifdef MULTI_LINE_NO_SEPARATOR
+						str = "";
+#else
+						if (cols_height [i] == 0)
+							str = "";
+						else
+							str = ".";
+						align_center = TRUE;
+#endif
+					}
+					
+					if (cols_is_num [i])
+						g_string_append_printf (string, "%*s", cols_size [i], str);
+					else {
+						gint j, max;
+						if (str) 
+							max = cols_size [i] - g_utf8_strlen (str, -1);
+						else
+							max = cols_size [i];
+
+						if (!align_center) {
+							g_string_append_printf (string, "%s", str);
+							for (j = 0; j < max; j++)
+								g_string_append_c (string, ' ');
+						}
+						else {
+							for (j = 0; j < max / 2; j++)
+								g_string_append_c (string, ' ');
+							g_string_append_printf (string, "%s", str);
+							for ( j+= g_utf8_strlen (str, -1); j < cols_size [i]; j++)
+								g_string_append_c (string, ' ');
+						}
+						/*
+						gint j, max;
+						if (str) {
+							g_string_append_printf (string, "%s", str);
+							max = cols_size [i] - g_utf8_strlen (str, -1);
+						}
+						else
+							max = cols_size [i];
+						for (j = 0; j < max; j++)
+							g_string_append_c (string, ' ');
+						*/
+					}
+				}
+				g_string_append_c (string, '\n');
+			}
+
+			g_free (cols_height);
+			for (i = 0; i < n_cols; i++) 
+				g_strfreev (cols_str [i]);
+			g_free (cols_str);
 		}
 	}
 	else 
