@@ -2,20 +2,20 @@
  *
  * Copyright (C) 2005 - 2007 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This Library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this Library; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include <string.h>
@@ -104,6 +104,7 @@ enum
 	PROP_0,
 	PROP_MODEL,
 	PROP_ADD_NULL_ENTRY,
+	PROP_DEFER_SYNC
 };
 
 /*
@@ -128,7 +129,7 @@ typedef struct
 	RowModif      *row_modif;    /* RowModif in which this structure instance appears */
 	gint           model_column; /* column index in the GdaDataModel */
         GValue        *value;        /* can also be GDA_TYPE_LIST for multiple values; values are owned here */
-        GValue        *attributes;   /* holds a GUINT of GdaValueAttribute */
+        GValue        *attributes;   /* holds flags of GdaValueAttribute */
 } RowValue;
 #define ROW_VALUE(x) ((RowValue *)(x))
 
@@ -149,7 +150,7 @@ typedef struct
 struct _GdaDataProxyPrivate
 {
 	GdaDataModel      *model; /* Gda model which is proxied */
-	GValue           **columns_attrs; /* Each GValue holds a GUINT of GdaValueAttribute to proxy cols. attributes */
+	GValue           **columns_attrs; /* Each GValue holds a flag of GdaValueAttribute to proxy cols. attributes */
 
 	gint               model_nb_cols; /* = gda_data_model_get_n_columns (model) */
 	gboolean           notify_changes;
@@ -166,6 +167,7 @@ struct _GdaDataProxyPrivate
 
 	gboolean           add_null_entry; /* artificially add a NULL entry at the beginning of the tree model */
 
+	gboolean           defer_sync;
 	guint              idle_add_event_source; /* !=0 when data rows are being added */
 
 	/* chunking: ONLY accounting for @model's rows, not the new rows */
@@ -336,7 +338,7 @@ gda_data_proxy_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaDataProxyClass),
 			(GBaseInitFunc) NULL,
@@ -451,6 +453,9 @@ gda_data_proxy_class_init (GdaDataProxyClass *class)
 	g_object_class_install_property (object_class, PROP_ADD_NULL_ENTRY,
 					 g_param_spec_boolean ("prepend_null_entry", NULL, NULL, FALSE,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
+	g_object_class_install_property (object_class, PROP_DEFER_SYNC,
+					 g_param_spec_boolean ("defer_sync", NULL, NULL, TRUE,
+							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 }
 
 static void
@@ -494,6 +499,7 @@ gda_data_proxy_init (GdaDataProxy *proxy)
 	proxy->priv->proxy_has_changed = FALSE;
 
 	proxy->priv->add_null_entry = FALSE;
+	proxy->priv->defer_sync = TRUE;
 	proxy->priv->idle_add_event_source = 0;
 
 	proxy->priv->sample_first_row = 0;
@@ -650,9 +656,6 @@ gda_data_proxy_set_property (GObject *object,
 			g_object_ref (model);
 			gda_object_connect_destroy (GDA_OBJECT (model), G_CALLBACK (destroyed_object_cb), object);
 			
-			/* if (gda_data_model_get_status (model) & GDA_DATA_MODEL_NEEDS_INIT_REFRESH)  */
-			/* gda_data_model_refresh (model, NULL); */
-			
 			proxy->priv->model_nb_cols = gda_data_model_get_n_columns (model);
 			
 			/* column attributes */
@@ -687,6 +690,14 @@ gda_data_proxy_set_property (GObject *object,
 					gda_data_model_row_removed ((GdaDataModel *) proxy, 0);
 			}
 			break;
+		case PROP_DEFER_SYNC:
+			proxy->priv->defer_sync = g_value_get_boolean (value);
+			if (!proxy->priv->defer_sync && proxy->priv->idle_add_event_source) {
+				g_idle_remove_by_data (proxy);
+				proxy->priv->idle_add_event_source = 0;
+				while (idle_add_model_rows (proxy)) ;
+			}
+			break;
 		}
 	}
 }
@@ -705,6 +716,9 @@ gda_data_proxy_get_property (GObject *object,
 		case PROP_ADD_NULL_ENTRY:
 			g_value_set_boolean (value, proxy->priv->add_null_entry);
 			break;
+		case PROP_DEFER_SYNC:
+			g_value_set_boolean (value, proxy->priv->defer_sync);
+			break;
 		}
 	}
 }
@@ -723,6 +737,7 @@ proxied_model_data_changed_cb (GdaDataModel *model, GdaDataProxy *proxy)
 {
 	gint nb_new_rows = 0;
 	gint nb_cols;
+
 	if (proxy->priv->ignore_proxied_changes) {
 		proxy->priv->proxy_has_changed = TRUE;
 		return;
@@ -1105,6 +1120,9 @@ gda_data_proxy_delete (GdaDataProxy *proxy, gint proxy_row)
 	g_return_if_fail (proxy->priv);
 	g_return_if_fail (proxy_row >= 0);
 
+	if (! (gda_data_model_get_access_flags ((GdaDataModel*) proxy) & GDA_DATA_MODEL_ACCESS_DELETE))
+		return;
+
 	model_row = proxy_row_to_model_row (proxy, proxy_row);
 	rm = find_row_modify_for_proxy_row (proxy, proxy_row);
 	if (rm) {
@@ -1262,6 +1280,7 @@ gda_data_proxy_find_row_from_values (GdaDataProxy *proxy, GSList *values,
 	 */
 	if (proxy->priv->idle_add_event_source) {
 		g_idle_remove_by_data (proxy);
+		proxy->priv->idle_add_event_source = 0;
 		while (idle_add_model_rows (proxy)) ;
 	}
 
@@ -1331,6 +1350,9 @@ gda_data_proxy_append (GdaDataProxy *proxy)
 	g_return_val_if_fail (GDA_IS_DATA_PROXY (proxy), -1);
 	g_return_val_if_fail (proxy->priv, -1);
 
+	if (! (gda_data_model_get_access_flags ((GdaDataModel *) proxy) & GDA_DATA_MODEL_ACCESS_INSERT)) 
+		return -1;
+
 	/* RowModif structure */
 	rm = row_modifs_new (proxy, -1);
 	rm->model_row = -1;
@@ -1356,7 +1378,7 @@ gda_data_proxy_append (GdaDataProxy *proxy)
 		rv = g_new0 (RowValue, 1);
 		rv->row_modif = rm;
 		rv->model_column = col;
-		rv->attributes = gda_value_new (G_TYPE_UINT);
+		rv->attributes = gda_value_new (GDA_TYPE_VALUE_ATTRIBUTE);
 		rv->value = NULL;
 		rm->modify_values = g_slist_prepend (rm->modify_values, rv);	
 		
@@ -1618,28 +1640,32 @@ commit_row_modif (GdaDataProxy *proxy, RowModif *rm, GError **error)
 				/* Don't yet emit a "delete" signal for that row, but increment 
 				 * proxy->priv->deferred_nb_rows
 				 */
+				gint index = find_proxy_row_for_row_modif (proxy, rm);
+
 				proxy->priv->all_modifs = g_slist_remove (proxy->priv->all_modifs, rm);
 				proxy->priv->new_rows = g_slist_remove (proxy->priv->new_rows, rm);
 				row_modifs_free (rm);
 
 				proxy->priv->deferred_nb_rows ++;
 				freedone = TRUE;
+				/* signal row actually changed */
+				g_signal_emit (G_OBJECT (proxy),
+					       gda_data_proxy_signals[POST_CHANGES_APPLIED],
+					       0, index, -1);
 			}
 		}
 	}
 
-	if (!err) {
+	if (!err && !freedone) {
 		/* signal row actually changed */
 		g_signal_emit (G_OBJECT (proxy),
 			       gda_data_proxy_signals[POST_CHANGES_APPLIED],
 			       0, find_proxy_row_for_row_modif (proxy, rm), rm->model_row);
-	
-		if (!freedone) {
-			/* get rid of the commited change, where rm->model_row >= 0 */
-			proxy->priv->all_modifs = g_slist_remove (proxy->priv->all_modifs, rm);
-			g_hash_table_remove (proxy->priv->modify_rows, GINT_TO_POINTER (rm->model_row));
-			row_modifs_free (rm);
-		}
+		
+		/* get rid of the commited change, where rm->model_row >= 0 */
+		proxy->priv->all_modifs = g_slist_remove (proxy->priv->all_modifs, rm);
+		g_hash_table_remove (proxy->priv->modify_rows, GINT_TO_POINTER (rm->model_row));
+		row_modifs_free (rm);
 	}
 
 	proxy->priv->ignore_proxied_changes = ignore_proxied_changes;
@@ -1977,6 +2003,12 @@ adjust_displayed_chunck (GdaDataProxy *proxy)
 	gda_object_unblock_changed (GDA_OBJECT (proxy));
 	gda_data_model_signal_emit_changed ((GdaDataModel *) proxy);
 
+	if (!proxy->priv->defer_sync && proxy->priv->idle_add_event_source) {
+		g_idle_remove_by_data (proxy);
+		proxy->priv->idle_add_event_source = 0;
+		while (idle_add_model_rows (proxy)) ;
+	}
+
 #ifdef GDA_DEBUG_NO
 	g_print ("//GdaDataProxy adjusted sample: %d<->%d (=%d / %d rows, %s)\n", proxy->priv->sample_first_row, 
 		 proxy->priv->sample_last_row,
@@ -2085,6 +2117,8 @@ gda_data_proxy_apply_all_changes (GdaDataProxy *proxy, GError **error)
 	proxy->priv->ignore_proxied_changes = FALSE;
 
 	gda_data_model_send_hint (proxy->priv->model, GDA_DATA_MODEL_HINT_END_BATCH_UPDATE, NULL);
+	if (proxy->priv->proxy_has_changed) 
+		adjust_displayed_chunck (proxy);
 
 	return allok;
 }
@@ -2321,9 +2355,7 @@ gda_data_proxy_create_iter (GdaDataModel *model)
 	proxy = GDA_DATA_PROXY (model);
 	g_return_val_if_fail (proxy->priv, FALSE);
 
-	iter = (GdaDataModelIter *) g_object_new (GDA_TYPE_DATA_MODEL_ITER, 
-						  "dict", gda_object_get_dict (GDA_OBJECT (proxy->priv->model)), 
-						  "data_model", proxy->priv->model, NULL);
+	iter = gda_data_model_create_iter (proxy->priv->model);
 	g_object_set (G_OBJECT (iter), "forced_model", proxy, NULL);
 
 	iter2 = gda_data_model_create_iter (proxy->priv->model);
@@ -2341,8 +2373,10 @@ gda_data_proxy_create_iter (GdaDataModel *model)
 				g_free (plugin);
 			}
 		}
-		if (plist1 || plist2)
-			g_warning ("Proxy iterator does not have the same length as proxied model's iterator");
+		if (plist1 || plist2) 
+			g_warning ("Proxy iterator does not have the same length as proxied model's iterator: %d/%d",
+				   g_slist_length (GDA_PARAMETER_LIST (iter)->parameters),
+				   g_slist_length (GDA_PARAMETER_LIST (iter2)->parameters));
 		g_object_unref (iter2);
 	}
 
@@ -2377,8 +2411,11 @@ gda_data_proxy_set_value_at (GdaDataModel *model, gint col, gint proxy_row, cons
 	g_return_val_if_fail (proxy_row >= 0, FALSE);
 
 	att = gda_data_proxy_get_value_attributes (proxy, proxy_row, col);
-	if (att & GDA_VALUE_ATTR_NO_MODIF)
+	if (att & GDA_VALUE_ATTR_NO_MODIF) {
+		g_set_error (error, GDA_DATA_PROXY_ERROR, 0,
+			     _("Value is read-only"));
 		return FALSE;
+	}
 
 	row = proxy_row;
 	if ((row == 0) && proxy->priv->add_null_entry) {
