@@ -53,20 +53,20 @@
 
 /* VOID:OBJECT (gda-marshal.list:27) */
 
-/* VOID:POINTER,POINTER (gda-marshal.list:28) */
+/* VOID:OBJECT,OBJECT (gda-marshal.list:28) */
 void
-gda_marshal_VOID__POINTER_POINTER (GClosure     *closure,
-                                   GValue       *return_value,
-                                   guint         n_param_values,
-                                   const GValue *param_values,
-                                   gpointer      invocation_hint,
-                                   gpointer      marshal_data)
+gda_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
+                                 GValue       *return_value,
+                                 guint         n_param_values,
+                                 const GValue *param_values,
+                                 gpointer      invocation_hint,
+                                 gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__POINTER_POINTER) (gpointer     data1,
-                                                      gpointer     arg_1,
-                                                      gpointer     arg_2,
-                                                      gpointer     data2);
-  register GMarshalFunc_VOID__POINTER_POINTER callback;
+  typedef void (*GMarshalFunc_VOID__OBJECT_OBJECT) (gpointer     data1,
+                                                    gpointer     arg_1,
+                                                    gpointer     arg_2,
+                                                    gpointer     data2);
+  register GMarshalFunc_VOID__OBJECT_OBJECT callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
 
@@ -82,29 +82,29 @@ gda_marshal_VOID__POINTER_POINTER (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_VOID__POINTER_POINTER) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_VOID__OBJECT_OBJECT) (marshal_data ? marshal_data : cc->callback);
 
   callback (data1,
-            g_marshal_value_peek_pointer (param_values + 1),
-            g_marshal_value_peek_pointer (param_values + 2),
+            g_marshal_value_peek_object (param_values + 1),
+            g_marshal_value_peek_object (param_values + 2),
             data2);
 }
 
-/* VOID:POINTER,INT,POINTER (gda-marshal.list:29) */
+/* VOID:OBJECT,ENUM,OBJECT (gda-marshal.list:29) */
 void
-gda_marshal_VOID__POINTER_INT_POINTER (GClosure     *closure,
-                                       GValue       *return_value,
-                                       guint         n_param_values,
-                                       const GValue *param_values,
-                                       gpointer      invocation_hint,
-                                       gpointer      marshal_data)
+gda_marshal_VOID__OBJECT_ENUM_OBJECT (GClosure     *closure,
+                                      GValue       *return_value,
+                                      guint         n_param_values,
+                                      const GValue *param_values,
+                                      gpointer      invocation_hint,
+                                      gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__POINTER_INT_POINTER) (gpointer     data1,
-                                                          gpointer     arg_1,
-                                                          gint         arg_2,
-                                                          gpointer     arg_3,
-                                                          gpointer     data2);
-  register GMarshalFunc_VOID__POINTER_INT_POINTER callback;
+  typedef void (*GMarshalFunc_VOID__OBJECT_ENUM_OBJECT) (gpointer     data1,
+                                                         gpointer     arg_1,
+                                                         gint         arg_2,
+                                                         gpointer     arg_3,
+                                                         gpointer     data2);
+  register GMarshalFunc_VOID__OBJECT_ENUM_OBJECT callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
 
@@ -120,29 +120,29 @@ gda_marshal_VOID__POINTER_INT_POINTER (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_VOID__POINTER_INT_POINTER) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_VOID__OBJECT_ENUM_OBJECT) (marshal_data ? marshal_data : cc->callback);
 
   callback (data1,
-            g_marshal_value_peek_pointer (param_values + 1),
-            g_marshal_value_peek_int (param_values + 2),
-            g_marshal_value_peek_pointer (param_values + 3),
+            g_marshal_value_peek_object (param_values + 1),
+            g_marshal_value_peek_enum (param_values + 2),
+            g_marshal_value_peek_object (param_values + 3),
             data2);
 }
 
-/* VOID:POINTER,UINT (gda-marshal.list:30) */
+/* VOID:OBJECT,UINT (gda-marshal.list:30) */
 void
-gda_marshal_VOID__POINTER_UINT (GClosure     *closure,
-                                GValue       *return_value,
-                                guint         n_param_values,
-                                const GValue *param_values,
-                                gpointer      invocation_hint,
-                                gpointer      marshal_data)
+gda_marshal_VOID__OBJECT_UINT (GClosure     *closure,
+                               GValue       *return_value,
+                               guint         n_param_values,
+                               const GValue *param_values,
+                               gpointer      invocation_hint,
+                               gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__POINTER_UINT) (gpointer     data1,
-                                                   gpointer     arg_1,
-                                                   guint        arg_2,
-                                                   gpointer     data2);
-  register GMarshalFunc_VOID__POINTER_UINT callback;
+  typedef void (*GMarshalFunc_VOID__OBJECT_UINT) (gpointer     data1,
+                                                  gpointer     arg_1,
+                                                  guint        arg_2,
+                                                  gpointer     data2);
+  register GMarshalFunc_VOID__OBJECT_UINT callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
 
@@ -158,29 +158,29 @@ gda_marshal_VOID__POINTER_UINT (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_VOID__POINTER_UINT) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_VOID__OBJECT_UINT) (marshal_data ? marshal_data : cc->callback);
 
   callback (data1,
-            g_marshal_value_peek_pointer (param_values + 1),
+            g_marshal_value_peek_object (param_values + 1),
             g_marshal_value_peek_uint (param_values + 2),
             data2);
 }
 
-/* VOID:POINTER,UINT,UINT (gda-marshal.list:31) */
+/* VOID:OBJECT,UINT,UINT (gda-marshal.list:31) */
 void
-gda_marshal_VOID__POINTER_UINT_UINT (GClosure     *closure,
-                                     GValue       *return_value,
-                                     guint         n_param_values,
-                                     const GValue *param_values,
-                                     gpointer      invocation_hint,
-                                     gpointer      marshal_data)
+gda_marshal_VOID__OBJECT_UINT_UINT (GClosure     *closure,
+                                    GValue       *return_value,
+                                    guint         n_param_values,
+                                    const GValue *param_values,
+                                    gpointer      invocation_hint,
+                                    gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__POINTER_UINT_UINT) (gpointer     data1,
-                                                        gpointer     arg_1,
-                                                        guint        arg_2,
-                                                        guint        arg_3,
-                                                        gpointer     data2);
-  register GMarshalFunc_VOID__POINTER_UINT_UINT callback;
+  typedef void (*GMarshalFunc_VOID__OBJECT_UINT_UINT) (gpointer     data1,
+                                                       gpointer     arg_1,
+                                                       guint        arg_2,
+                                                       guint        arg_3,
+                                                       gpointer     data2);
+  register GMarshalFunc_VOID__OBJECT_UINT_UINT callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
 
@@ -196,29 +196,68 @@ gda_marshal_VOID__POINTER_UINT_UINT (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_VOID__POINTER_UINT_UINT) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_VOID__OBJECT_UINT_UINT) (marshal_data ? marshal_data : cc->callback);
 
   callback (data1,
-            g_marshal_value_peek_pointer (param_values + 1),
+            g_marshal_value_peek_object (param_values + 1),
             g_marshal_value_peek_uint (param_values + 2),
             g_marshal_value_peek_uint (param_values + 3),
             data2);
 }
 
-/* VOID:ENUM,POINTER (gda-marshal.list:32) */
+/* VOID:STRING,UINT,UINT (gda-marshal.list:32) */
 void
-gda_marshal_VOID__ENUM_POINTER (GClosure     *closure,
-                                GValue       *return_value,
-                                guint         n_param_values,
-                                const GValue *param_values,
-                                gpointer      invocation_hint,
-                                gpointer      marshal_data)
+gda_marshal_VOID__STRING_UINT_UINT (GClosure     *closure,
+                                    GValue       *return_value,
+                                    guint         n_param_values,
+                                    const GValue *param_values,
+                                    gpointer      invocation_hint,
+                                    gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__ENUM_POINTER) (gpointer     data1,
-                                                   gint         arg_1,
-                                                   gpointer     arg_2,
-                                                   gpointer     data2);
-  register GMarshalFunc_VOID__ENUM_POINTER callback;
+  typedef void (*GMarshalFunc_VOID__STRING_UINT_UINT) (gpointer     data1,
+                                                       gpointer     arg_1,
+                                                       guint        arg_2,
+                                                       guint        arg_3,
+                                                       gpointer     data2);
+  register GMarshalFunc_VOID__STRING_UINT_UINT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 4);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__STRING_UINT_UINT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_string (param_values + 1),
+            g_marshal_value_peek_uint (param_values + 2),
+            g_marshal_value_peek_uint (param_values + 3),
+            data2);
+}
+
+/* VOID:ENUM,OBJECT (gda-marshal.list:33) */
+void
+gda_marshal_VOID__ENUM_OBJECT (GClosure     *closure,
+                               GValue       *return_value,
+                               guint         n_param_values,
+                               const GValue *param_values,
+                               gpointer      invocation_hint,
+                               gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__ENUM_OBJECT) (gpointer     data1,
+                                                  gint         arg_1,
+                                                  gpointer     arg_2,
+                                                  gpointer     data2);
+  register GMarshalFunc_VOID__ENUM_OBJECT callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
 
@@ -234,17 +273,17 @@ gda_marshal_VOID__ENUM_POINTER (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_VOID__ENUM_POINTER) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_VOID__ENUM_OBJECT) (marshal_data ? marshal_data : cc->callback);
 
   callback (data1,
             g_marshal_value_peek_enum (param_values + 1),
-            g_marshal_value_peek_pointer (param_values + 2),
+            g_marshal_value_peek_object (param_values + 2),
             data2);
 }
 
-/* VOID:UINT,POINTER (gda-marshal.list:33) */
+/* VOID:UINT,POINTER (gda-marshal.list:34) */
 
-/* VOID:INT,INT (gda-marshal.list:34) */
+/* VOID:INT,INT (gda-marshal.list:35) */
 void
 gda_marshal_VOID__INT_INT (GClosure     *closure,
                            GValue       *return_value,
@@ -281,21 +320,21 @@ gda_marshal_VOID__INT_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:INT,POINTER,POINTER (gda-marshal.list:35) */
+/* VOID:INT,BOXED,BOXED (gda-marshal.list:36) */
 void
-gda_marshal_VOID__INT_POINTER_POINTER (GClosure     *closure,
-                                       GValue       *return_value,
-                                       guint         n_param_values,
-                                       const GValue *param_values,
-                                       gpointer      invocation_hint,
-                                       gpointer      marshal_data)
+gda_marshal_VOID__INT_BOXED_BOXED (GClosure     *closure,
+                                   GValue       *return_value,
+                                   guint         n_param_values,
+                                   const GValue *param_values,
+                                   gpointer      invocation_hint,
+                                   gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__INT_POINTER_POINTER) (gpointer     data1,
-                                                          gint         arg_1,
-                                                          gpointer     arg_2,
-                                                          gpointer     arg_3,
-                                                          gpointer     data2);
-  register GMarshalFunc_VOID__INT_POINTER_POINTER callback;
+  typedef void (*GMarshalFunc_VOID__INT_BOXED_BOXED) (gpointer     data1,
+                                                      gint         arg_1,
+                                                      gpointer     arg_2,
+                                                      gpointer     arg_3,
+                                                      gpointer     data2);
+  register GMarshalFunc_VOID__INT_BOXED_BOXED callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
 
@@ -311,16 +350,16 @@ gda_marshal_VOID__INT_POINTER_POINTER (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_VOID__INT_POINTER_POINTER) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_VOID__INT_BOXED_BOXED) (marshal_data ? marshal_data : cc->callback);
 
   callback (data1,
             g_marshal_value_peek_int (param_values + 1),
-            g_marshal_value_peek_pointer (param_values + 2),
-            g_marshal_value_peek_pointer (param_values + 3),
+            g_marshal_value_peek_boxed (param_values + 2),
+            g_marshal_value_peek_boxed (param_values + 3),
             data2);
 }
 
-/* VOID:INT,BOOLEAN (gda-marshal.list:36) */
+/* VOID:INT,BOOLEAN (gda-marshal.list:37) */
 void
 gda_marshal_VOID__INT_BOOLEAN (GClosure     *closure,
                                GValue       *return_value,
@@ -357,7 +396,7 @@ gda_marshal_VOID__INT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,INT (gda-marshal.list:37) */
+/* VOID:STRING,INT (gda-marshal.list:38) */
 void
 gda_marshal_VOID__STRING_INT (GClosure     *closure,
                               GValue       *return_value,
@@ -394,7 +433,7 @@ gda_marshal_VOID__STRING_INT (GClosure     *closure,
             data2);
 }
 
-/* BOOLEAN:INT (gda-marshal.list:38) */
+/* BOOLEAN:INT (gda-marshal.list:39) */
 void
 gda_marshal_BOOLEAN__INT (GClosure     *closure,
                           GValue       *return_value,
@@ -433,3 +472,44 @@ gda_marshal_BOOLEAN__INT (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
+/* BOOLEAN:INT,INT (gda-marshal.list:40) */
+void
+gda_marshal_BOOLEAN__INT_INT (GClosure     *closure,
+                              GValue       *return_value,
+                              guint         n_param_values,
+                              const GValue *param_values,
+                              gpointer      invocation_hint,
+                              gpointer      marshal_data)
+{
+  typedef gboolean (*GMarshalFunc_BOOLEAN__INT_INT) (gpointer     data1,
+                                                     gint         arg_1,
+                                                     gint         arg_2,
+                                                     gpointer     data2);
+  register GMarshalFunc_BOOLEAN__INT_INT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  gboolean v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_BOOLEAN__INT_INT) (marshal_data ? marshal_data : cc->callback);
+
+  v_return = callback (data1,
+                       g_marshal_value_peek_int (param_values + 1),
+                       g_marshal_value_peek_int (param_values + 2),
+                       data2);
+
+  g_value_set_boolean (return_value, v_return);
+}
+
