@@ -2,20 +2,20 @@
  *
  * Copyright (C) 2003 - 2006 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This Library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this Library; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include <string.h>
@@ -89,7 +89,7 @@ gda_object_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaObjectClass),
 			(GBaseInitFunc) NULL,
@@ -104,6 +104,7 @@ gda_object_get_type (void)
 		
 		type = g_type_register_static (G_TYPE_OBJECT, "GdaObject", &info, 0);
 	}
+
 	return type;
 }
 
@@ -189,6 +190,7 @@ gda_object_class_init (GdaObjectClass * class)
 
 	/* class attributes */
 	class->id_unique_enforced = TRUE;
+	class->name_as_initial_id = FALSE;
 
 	/* Properties */
 	object_class->set_property = gda_object_set_property;
@@ -433,6 +435,13 @@ gda_object_set_name (GdaObject *gdaobj, const gchar *name)
 		else
 			changed = TRUE;
 		gdaobj->priv->name = g_strdup (name);
+
+		GdaObjectClass *real_class;
+		real_class = (GdaObjectClass *) G_OBJECT_GET_CLASS (gdaobj);
+		if (real_class->name_as_initial_id) {
+			if (!gdaobj->priv->id_string)
+				gda_object_set_id (gdaobj, name);
+		}
 	}
 
 	if (changed) {
@@ -671,9 +680,11 @@ gda_object_destroy_check (GdaObject *gdaobj)
 /**
  * gda_object_connect_destroy
  * @gdaobj: a #GdaObject object
+ * @callback:
+ * @data:
  * 
- * Connects the "destoy" signal of the @gdaobj object but first cheks that @gdaobj
- * exists and has not yet been destroyed.
+ * Connects the "destroy" signal of the @gdaobj object but first cheks that
+ * @gdaobj exists and has not yet been destroyed.
  *
  * Returns: the handler id of the signal
  */
