@@ -1554,16 +1554,8 @@ parsed_create_value_query_field (GdaQuery *query, gboolean add_to_query, ParseDa
 		list = g_list_next (list);
 	}
 
-	if (!real_type && (gdatype == G_TYPE_INVALID) && unspecvalue) {
-		if (error && !*error)
-			g_set_error (error,
-				     GDA_QUERY_ERROR,
-				     GDA_QUERY_SQL_ANALYSE_ERROR,
-				     _("Data type must be specified when no default value is provided, using G_TYPE_STRING"));
-		else
-			g_warning (_("Data type must be specified when no default value is provided, using G_TYPE_STRING"));
+	if (!real_type && (gdatype == G_TYPE_INVALID) && unspecvalue) 
 		gdatype = G_TYPE_STRING;
-	}
 
 	if (!real_type && (gdatype == G_TYPE_INVALID)) {
 		if (prov) {
@@ -1698,6 +1690,7 @@ parsed_create_value_query_field (GdaQuery *query, gboolean add_to_query, ParseDa
 	 * Using the param spec to set attributes
 	 */
 	isparam = g_list_length (param_specs) > 0 ? TRUE : FALSE;
+	gda_query_field_value_set_not_null (GDA_QUERY_FIELD_VALUE (qfield), TRUE);
 	list = param_specs;
 	while (list) {
 		param_spec *pspec = (param_spec*) list->data;
