@@ -63,7 +63,8 @@ typedef enum
 	GDA_QUERY_NO_CNC_ERROR,
 	GDA_QUERY_CNC_CLOSED_ERROR,
 	GDA_QUERY_EXEC_ERROR,
-	GDA_QUERY_PARAM_TYPE_ERROR
+	GDA_QUERY_PARAM_TYPE_ERROR,
+	GDA_QUERY_MULTIPLE_STATEMENTS_ERROR,
 } GdaQueryError;
 
 
@@ -96,7 +97,7 @@ struct _GdaQueryClass
 	void   (*sub_query_updated)    (GdaQuery *query, GdaQuery *sub_query);
 };
 
-GType              gda_query_get_type               (void);
+GType              gda_query_get_type               (void) G_GNUC_CONST;
 GdaQuery          *gda_query_new                    (GdaDict *dict);
 GdaQuery          *gda_query_new_copy               (GdaQuery *orig, GHashTable *replacements);
 GdaQuery          *gda_query_new_from_sql           (GdaDict *dict, const gchar *sql, GError **error);
@@ -120,7 +121,7 @@ GdaObject         *gda_query_execute                (GdaQuery *query, GdaParamet
 
 /* if SQL queries */
 void               gda_query_set_sql_text           (GdaQuery *query, const gchar *sql, GError **error);
-const gchar       *gda_query_get_sql_text           (GdaQuery *query);
+gchar             *gda_query_get_sql_text           (GdaQuery *query);
 
 /* for other types of queries */
 GSList            *gda_query_get_sub_queries        (GdaQuery *query);
