@@ -2,20 +2,20 @@
  *
  * Copyright (C) 2003 - 2007 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This Library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this Library; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 
@@ -39,8 +39,9 @@ typedef enum {
 	GDA_RENDERER_PARAMS_AS_DETAILED     = 1 << 1,
 	GDA_RENDERER_ERROR_IF_DEFAULT       = 1 << 2,
 	GDA_RENDERER_FIELDS_NO_TARGET_ALIAS = 1 << 3,
-	GDA_RENDERER_PARAMS_AS_COLON        = 1 << 4,/* params as :param_name */
-	GDA_RENDERER_PARAMS_AS_DOLLAR       = 1 << 5 /* params as $1, $2, etc (starts at $1) */
+	GDA_RENDERER_PARAMS_AS_COLON        = 1 << 4,/* params as :param_name, replacing any char not in [0-9A-Za-z] by '_' */
+	GDA_RENDERER_PARAMS_AS_DOLLAR       = 1 << 5,/* params as $1, $2, etc (starts at $1) */
+	GDA_RENDERER_PARAMS_AS_QMARK        = 1 << 6 /* params as ?1, ?2, etc (starts at ?1) */
 } GdaRendererOptions;
 
 /* struct for the interface */
