@@ -119,7 +119,7 @@ gda_server_provider_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaServerProviderClass),
 			(GBaseInitFunc) NULL,
@@ -133,6 +133,7 @@ gda_server_provider_get_type (void)
 		};
 		type = g_type_register_static (PARENT_TYPE, "GdaServerProvider", &info, G_TYPE_FLAG_ABSTRACT);
 	}
+
 	return type;
 }
 
@@ -193,6 +194,98 @@ gda_server_provider_get_info (GdaServerProvider *provider, GdaConnection *cnc)
 	return retval;
 }
 
+/**
+ * gda_server_provider_create_connection
+ * @client: a #GdaClient object to which the returned connection will be attached
+ * @provider: a #GdaServerProvider object.
+ * @dsn: a DSN string.
+ * @params:
+ * @username: user name for logging in.
+ * @password: password for authentication.
+ * @options: options for the connection (see #GdaConnectionOptions).
+ *
+ * Requests that the @provider creates a new #GdaConnection connection object.
+ * See gda_client_open_connection() for more information
+ *
+ * Returns: a newly-allocated #GdaConnection object, or NULL
+ * if it fails.
+ */
+GdaConnection *
+gda_server_provider_create_connection (GdaClient *client, GdaServerProvider *provider, 
+				       const gchar *dsn,
+				       const gchar *username,
+				       const gchar *password,
+				       GdaConnectionOptions options)
+{
+	GdaConnection *cnc;
+	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
+
+	if (CLASS (provider)->create_connection) {
+		cnc = CLASS (provider)->create_connection (provider);
+		if (cnc) {
+			g_object_set (G_OBJECT (cnc), "client", client, "provider_obj", provider, NULL);
+			if (dsn && *dsn)
+				g_object_set (G_OBJECT (cnc), "dsn", dsn, NULL);
+			g_object_set (G_OBJECT (cnc), 
+				      "username", username,
+				      "password", password,
+				      "options", options, NULL);
+		}
+	}
+	else 
+		cnc = gda_connection_new (client, provider, dsn, username, password, options);
+
+	return cnc;
+}
+
+/**
+ * gda_server_provider_create_connection_from_string
+ * @client: a #GdaClient object to which the returned connection will be attached
+ * @provider: a #GdaServerProvider object.
+ * @cnc_string: a connection string.
+ * @params:
+ * @username: user name for logging in.
+ * @password: password for authentication.
+ * @options: options for the connection (see #GdaConnectionOptions).
+ *
+ * Requests that the @provider creates a new #GdaConnection connection object.
+ * See gda_client_open_connection() for more information
+ *
+ * Returns: a newly-allocated #GdaConnection object, or NULL
+ * if it fails.
+ */
+GdaConnection *
+gda_server_provider_create_connection_from_string (GdaClient *client, GdaServerProvider *provider, 
+						   const gchar *cnc_string,
+						   const gchar *username,
+						   const gchar *password,
+						   GdaConnectionOptions options)
+{
+	GdaConnection *cnc;
+	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
+
+	if (CLASS (provider)->create_connection) {
+		cnc = CLASS (provider)->create_connection (provider);
+		if (cnc) {
+			g_object_set (G_OBJECT (cnc), "client", client, "provider_obj", provider, NULL);
+			if (cnc_string && *cnc_string)
+				g_object_set (G_OBJECT (cnc), "cnc_string", cnc_string, NULL);
+			g_object_set (G_OBJECT (cnc), 
+				      "username", username,
+				      "password", password,
+				      "options", options, NULL);
+		}
+	}
+	else 
+		cnc = (GdaConnection *) g_object_new (GDA_TYPE_CONNECTION, "client", client, "provider_obj", provider,
+						      "cnc-string", cnc_string,
+						      "username", username,
+						      "password", password,
+						      "options", options, NULL);
+
+	return cnc;
+}
+
 /**
  * gda_server_provider_open_connection
  * @provider: a #GdaServerProvider object.
@@ -204,8 +297,7 @@ gda_server_provider_get_info (GdaServerProvider *provider, GdaConnection *cnc)
  * Tries to open a new connection on the given #GdaServerProvider
  * object.
  *
- * Returns: a newly-allocated #GdaServerConnection object, or NULL
- * if it fails.
+ * Returns: TRUE if no error occurred
  */
 gboolean
 gda_server_provider_open_connection (GdaServerProvider *provider,
@@ -436,7 +528,7 @@ gda_server_provider_create_operation (GdaServerProvider *provider, GdaConnection
 				      GdaServerOperationType type, 
 				      GdaParameterList *options, GError **error)
 {
-	static OpReq **op_req_table = NULL;
+	OpReq **op_req_table = NULL;
 
 	if (! op_req_table) {
 		op_req_table = g_new0 (OpReq *, GDA_SERVER_OPERATION_NB);
@@ -614,6 +706,9 @@ gda_server_provider_perform_operation (GdaServerProvider *provider, GdaConnectio
  *   <listitem><para>%NULL is in the list if the correcponding statement could not be executed</para></listitem>
  * </itemizedlist>
  *
+ * See the documentation of the gda_connection_execute_command() for information
+ * about the @params list of parameters.
+ *
  * For every executed statement, a #GdaDataModel, a #GdaParameterList or %NULL will be appended to the returned list.
  * The returned list may contain fewer items than the @cmd command contained statements depending on the
  * options of @command (see gda_command_set_options()).
@@ -639,6 +734,7 @@ gda_server_provider_execute_command (GdaServerProvider *provider,
 	}
 #endif
 
+	gda_connection_clear_events_list (cnc);
 	return CLASS (provider)->execute_command (provider, cnc, cmd, params);
 }
 
@@ -694,6 +790,7 @@ gda_server_provider_execute_query (GdaServerProvider *provider,
 	}
 #endif
 	
+	gda_connection_clear_events_list (cnc);
 	return CLASS (provider)->execute_query (provider, cnc, query, params);
 }
 
@@ -993,7 +1090,7 @@ gda_server_provider_get_data_handler_dbms (GdaServerProvider *provider,
  *
  * Use @provider to create a new #GValue from a single string representation. 
  *
- * The @prefered_type can optionally ask @provider to return a #GValue of the requested type 
+ * The @prefered_type can optionnaly ask @provider to return a #GValue of the requested type 
  * (but if such a value can't be created from @string, then %NULL is returned); 
  * pass G_TYPE_INVALID if any returned type is acceptable.
  *
