@@ -1,21 +1,21 @@
 /* gda-threader.c
  *
- * Copyright (C) 2005 Vivien Malerba
+ * Copyright (C) 2005 - 2007 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This Library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this Library; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include <string.h>
@@ -69,7 +69,7 @@ gda_threader_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaThreaderClass),
 			(GBaseInitFunc) NULL,
@@ -84,6 +84,7 @@ gda_threader_get_type (void)
 		
 		type = g_type_register_static (G_TYPE_OBJECT, "GdaThreader", &info, 0);
 	}
+
 	return type;
 }
 
@@ -124,6 +125,8 @@ gda_threader_init (GdaThreader * thread)
 
 	if (! g_thread_supported()) 
 		g_warning ("You must initialize the multi threads environment using g_thread_init()");
+	if (getenv ("LIBGDA_NO_THREADS"))
+		g_warning ("The LIBGDA_NO_THREADS environment variable has been set, threading not supported");
 
 	thread->priv->next_job = 1;
 	thread->priv->nb_jobs = 0;
@@ -248,9 +251,9 @@ gda_threader_start_thread (GdaThreader *thread, GThreadFunc func, gpointer func_
 
 		g_hash_table_insert (thread->priv->jobs, GUINT_TO_POINTER (job->id), job);
 		if (! thread->priv->idle_func_id) 
-			thread->priv->idle_func_id = g_idle_add_full (150,
-								      (GSourceFunc) idle_catch_threads_end, 
-								      thread, NULL);
+			thread->priv->idle_func_id = g_timeout_add_full (G_PRIORITY_HIGH_IDLE, 150,
+									 (GSourceFunc) idle_catch_threads_end, 
+									 thread, NULL);
 		
 		return job->id;
 	}
