@@ -1,5 +1,5 @@
 /* GDA common library
- * Copyright (C) 1998 - 2006 The GNOME Foundation.
+ * Copyright (C) 1998 - 2007 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -41,6 +41,8 @@
 #include <locale.h>
 #endif
 
+#include <libgda/binreloc/gda-binreloc.h>
+
 /**
  * gda_g_type_to_string
  * @type: Type to convert from.
@@ -85,7 +87,7 @@ gda_g_type_from_string (const gchar *str)
  * @string: string to escape
  *
  * Escapes @string to make it understandable by a DBMS. The escape method is very common and replaces any
- * occurence of "'" with "''" and "\" with "\\"
+ * occurence of "'" with "\'" and "\" with "\\".
  */
 gchar *
 gda_default_escape_string (const gchar *string)
@@ -111,12 +113,7 @@ gda_default_escape_string (const gchar *string)
 	ret = g_new0 (gchar, size);
 	retptr = ret;
 	while (*ptr) {
-		if (*ptr == '\'') {
-			*retptr = '\'';
-			*(retptr+1) = *ptr;
-			retptr += 2;
-		}
-		else if (*ptr == '\\') {
+		if ((*ptr == '\'') || (*ptr == '\\')) {
 			*retptr = '\\';
 			*(retptr+1) = *ptr;
 			retptr += 2;
@@ -686,13 +683,10 @@ gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent,
 		
 			column = gda_data_model_describe_column (model, rcols [c]);
 			g_object_get (G_OBJECT (column), "id", &id, NULL);
-
 			if (id && *id)
 				col_ids [c] = g_strdup (id);
 			else
 				col_ids [c] = g_strdup_printf ("_%d", c);
-
-			g_free(id);
 		}
 	}
 
@@ -885,10 +879,9 @@ gda_utility_parameter_load_attributes (GdaParameter *param, xmlNodePtr node, GSL
 			
 			if (!isnull) {
 				GValue *value;
-				gchar* nodeval = (gchar*)xmlNodeGetContent (vnode);
 
 				value = g_new0 (GValue, 1);
-				if (! gda_value_set_from_string (value, nodeval, gdatype)) {
+				if (! gda_value_set_from_string (value, (gchar*)xmlNodeGetContent (vnode), gdatype)) {
 					/* error */
 					g_free (value);
 				}
@@ -896,8 +889,6 @@ gda_utility_parameter_load_attributes (GdaParameter *param, xmlNodePtr node, GSL
 					gda_parameter_set_value (param, value);
 					gda_value_free (value);
 				}
-
-				xmlFree(nodeval);
 			}
 			else {
 				gda_parameter_set_value (param, NULL);
@@ -964,3 +955,89 @@ gda_utility_find_or_create_data_type (GdaDict *dict, GdaServerProvider *prov, Gd
 
 	return dtype;
 }
+
+#define GDA_PARAM_ENCODE_TOKEN "__gda"
+/**
+ * gda_text_to_alphanum
+ * @text:
+ *
+ * The "encoding" consists in replacing non
+ * alphanumeric character with the string "__gdaXX" where XX is the hex. representation
+ * of the non alphanumeric char.
+ *
+ * Returns: a new string
+ */
+gchar *
+gda_text_to_alphanum (const gchar *text)
+{
+	GString *string;
+	const gchar* ptr = text;
+	gchar *ret;
+
+	/*g_print ("%s (%s) ", __FUNCTION__, text);*/
+	string = g_string_new ("");
+	for (ptr = text; ptr && *ptr; ptr++) {
+		if (! (((*ptr >= '0') && (*ptr <= '9')) ||
+		       ((*ptr >= 'A') && (*ptr <= 'Z')) ||
+		       ((*ptr >= 'a') && (*ptr <= 'z')))) {
+			g_string_append (string, GDA_PARAM_ENCODE_TOKEN);
+			g_string_append_printf (string, "%0x", *ptr);
+		}
+		else
+			g_string_append_c (string, *ptr);
+	}
+	ret = string->str;
+	g_string_free (string, FALSE);
+	/*g_print ("=>#%s#\n", ret);*/
+	return ret;
+}
+
+/**
+ * gda_alphanum_to_text
+ * @text:
+ *
+ * Does the opposite of gda_text_to_alphanum(), in the same string 
+ *
+ * Returns: @text if conversion succedded or %NULL if an error occurred
+ */
+gchar *
+gda_alphanum_to_text (gchar *text)
+{
+	gchar* ptr = text;
+	gint length = strlen (text);
+	static gint toklength = 0;
+
+	if (toklength == 0)
+		toklength = strlen (GDA_PARAM_ENCODE_TOKEN);
+	/*g_print ("%s (%s) ", __FUNCTION__, text);*/
+	for (ptr = text; ptr && *ptr; ) {
+		if ((length >= toklength + 2) && !strncmp (ptr, GDA_PARAM_ENCODE_TOKEN, toklength)) {
+			gchar *ptr2 = ptr + toklength;
+			char c = *ptr2;
+			gchar val;
+			if ((c >= 'a') && (c <= 'f'))
+				val = (c - 'a' + 10) * 16;
+			else if ((c >= '0') && (c <= '9'))
+				val = (c - '0') * 16;
+			else
+				return NULL;
+			c = *(ptr2+1);
+			if ((c >= 'a') && (c <= 'f'))
+				val += c - 'a' + 10;
+			else if ((c >= '0') && (c <= '9'))
+				val += c - '0';
+			else
+				return NULL;
+			*ptr = val;
+			ptr++;
+			length -= toklength + 1;
+			memmove (ptr, ptr + toklength + 1, length);
+		}
+		else {
+			ptr ++;
+			length --;
+		}
+	}
+	/*g_print ("=>#%s#\n", text);*/
+	return text;
+}
