@@ -86,6 +86,7 @@ gda_ldap_provider_class_init (GdaLdapProviderClass *klass)
 	provider_class->string_to_value = NULL;
 	provider_class->get_def_dbms_type = NULL;
 
+	provider_class->create_connection = NULL;
 	provider_class->open_connection = gda_ldap_provider_open_connection;
 	provider_class->close_connection = gda_ldap_provider_close_connection;
 	provider_class->get_database = gda_ldap_provider_get_database;
@@ -129,7 +130,7 @@ gda_ldap_provider_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static GTypeInfo info = {
 			sizeof (GdaLdapProviderClass),
 			(GBaseInitFunc) NULL,
