@@ -419,7 +419,7 @@ gda_ldap_recordset_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaLdapRecordsetClass),
 			(GBaseInitFunc) NULL,
