@@ -91,6 +91,7 @@ static void gda_msql_provider_class_init(GdaMsqlProviderClass *cl)
 	provider_class->string_to_value = NULL;
 	provider_class->get_def_dbms_type = NULL;
 
+	provider_class->create_connection = NULL;
 	provider_class->open_connection = gda_msql_provider_open_connection;
 	provider_class->close_connection = gda_msql_provider_close_connection;
 	provider_class->get_database = gda_msql_provider_get_database;
@@ -102,6 +103,7 @@ static void gda_msql_provider_class_init(GdaMsqlProviderClass *cl)
         provider_class->perform_operation = NULL;
 
 	provider_class->execute_command = gda_msql_provider_execute_command;
+	provider_class->execute_query = NULL;
 	provider_class->get_last_insert_id = NULL;
 
 	provider_class->begin_transaction = NULL;
@@ -110,9 +112,6 @@ static void gda_msql_provider_class_init(GdaMsqlProviderClass *cl)
 	provider_class->add_savepoint = NULL;
 	provider_class->rollback_savepoint = NULL;
 	provider_class->delete_savepoint = NULL;
-	
-	provider_class->create_blob = NULL;
-	provider_class->fetch_blob = NULL;	
 }
 
 static void 
@@ -135,7 +134,7 @@ GType gda_msql_provider_get_type(void)
 {
 	static GType type=0;
   
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static GTypeInfo info = {
 			sizeof(GdaMsqlProviderClass),
 			(GBaseInitFunc)NULL,
@@ -268,6 +267,11 @@ process_sql_commands (GList *rl,GdaConnection *cnc,
 			gint rc;
 			m_result *res;
 			GdaMsqlRecordset *rs;
+			GdaConnectionEvent *event;
+
+			event = gda_connection_event_new (GDA_CONNECTION_EVENT_COMMAND);
+			gda_connection_event_set_description (event, arr[n]);
+			gda_connection_add_event (cnc, event);
 
 			rc=msqlQuery(*sock,arr[n]);
 			if (rc<0) {
@@ -352,7 +356,7 @@ gda_msql_provider_execute_command(GdaServerProvider *p,
 	if (!cmd) return NULL;  
 	switch (gda_command_get_command_type(cmd)) {
 	case GDA_COMMAND_TYPE_SQL:
-		rl=process_sql_commands(rl,cnc,gda_command_get_text(cmd));
+		rl = process_sql_commands(rl,cnc,gda_command_get_text(cmd));
 		break;
 	case GDA_COMMAND_TYPE_TABLE:
 		str = g_strdup_printf("SELECT * FROM %s",gda_command_get_text(cmd));
