@@ -1,6 +1,6 @@
 
 /* GDA MySQL provider
- * Copyright (C) 1998 - 2006 The GNOME Foundation.
+ * Copyright (C) 1998 - 2007 The GNOME Foundation.
  *
  * AUTHORS:
  *      Michael Lausch <michael@lausch.at>
@@ -47,6 +47,7 @@
 
 #include <libgda/sql-delimiter/gda-sql-delimiter.h>
 #include <libgda/gda-connection-private.h>
+#include <libgda/binreloc/gda-binreloc.h>
 
 #define PARENT_TYPE GDA_TYPE_SERVER_PROVIDER
 
@@ -155,6 +156,7 @@ gda_mysql_provider_class_init (GdaMysqlProviderClass *klass)
 	provider_class->string_to_value = NULL;
 	provider_class->get_def_dbms_type = gda_mysql_provider_get_default_dbms_type;
 
+	provider_class->create_connection = NULL;
 	provider_class->open_connection = gda_mysql_provider_open_connection;
 	provider_class->close_connection = gda_mysql_provider_close_connection;
 	provider_class->get_database = gda_mysql_provider_get_database;
@@ -198,7 +200,7 @@ gda_mysql_provider_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static GTypeInfo info = {
 			sizeof (GdaMysqlProviderClass),
 			(GBaseInitFunc) NULL,
@@ -466,7 +468,7 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql)
 			}
 
 			/* execute the command on the server */
-			rc = mysql_real_query (mysql, arr[n], strlen (arr[n]));
+			rc = gda_mysql_real_query_wrap (cnc, mysql, arr[n], strlen (arr[n]));
 			if (rc != 0) {
 				error = gda_mysql_make_error (mysql);
 				gda_connection_add_event (cnc, error);
@@ -523,6 +525,7 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql)
 						str = g_strdup_printf ("%s %d", tmp, changes);
 					gda_connection_event_set_description (event, str);
 					g_free (str);
+					g_free (tmp);
 					gda_connection_add_event (cnc, event);
 				}
 			}
@@ -617,14 +620,18 @@ gda_mysql_provider_create_operation (GdaServerProvider *provider, GdaConnection
 	gchar *file;
 	GdaServerOperation *op;
 	gchar *str;
+	gchar *dir;
 	
 	file = g_utf8_strdown (gda_server_operation_op_type_to_string (type), -1);
 	str = g_strdup_printf ("mysql_specs_%s.xml", file);
 	g_free (file);
-	file = g_build_filename (LIBGDA_DATA_DIR, str, NULL);
+
+	dir = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, NULL);
+	file = gda_server_provider_find_file (provider, dir, str);
+	g_free (dir);
 	g_free (str);
 
-	if (! g_file_test (file, G_FILE_TEST_EXISTS)) {
+	if (! file) {
 		g_set_error (error, 0, 0, _("Missing spec. file '%s'"), file);
 		return NULL;
 	}
@@ -642,14 +649,18 @@ gda_mysql_provider_render_operation (GdaServerProvider *provider, GdaConnection
 	gchar *sql = NULL;
 	gchar *file;
 	gchar *str;
-	
+	gchar *dir;
+
 	file = g_utf8_strdown (gda_server_operation_op_type_to_string (gda_server_operation_get_op_type (op)), -1);
 	str = g_strdup_printf ("mysql_specs_%s.xml", file);
 	g_free (file);
-	file = g_build_filename (LIBGDA_DATA_DIR, str, NULL);
+
+	dir = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, NULL);
+	file = gda_server_provider_find_file (provider, dir, str);
+	g_free (dir);
 	g_free (str);
 
-	if (! g_file_test (file, G_FILE_TEST_EXISTS)) {
+	if (! file) {
 		g_set_error (error, 0, 0, _("Missing spec. file '%s'"), file);
 		return NULL;
 	}
@@ -747,6 +758,11 @@ gda_mysql_provider_perform_operation (GdaServerProvider *provider, GdaConnection
 			sql = gda_server_provider_render_operation (provider, cnc, op, error);
 			if (!sql)
 				return FALSE;
+
+			GdaConnectionEvent *event = gda_connection_event_new (GDA_CONNECTION_EVENT_COMMAND);
+			gda_connection_event_set_description (event, sql);
+			gda_connection_add_event (cnc, event);
+
 			res = mysql_query (mysql, sql);
 			g_free (sql);
 			
@@ -831,6 +847,12 @@ gda_mysql_provider_get_last_insert_id (GdaServerProvider *provider,
 	g_return_val_if_fail (GDA_IS_MYSQL_PROVIDER (myprv), NULL);
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
+	if (recset) {
+		g_return_val_if_fail (GDA_IS_MYSQL_RECORDSET (recset), NULL);
+		TO_IMPLEMENT;
+		return NULL;
+	}
+
 	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
 	if (!mysql) {
 		gda_connection_add_event_string (cnc, _("Invalid MYSQL handle"));
@@ -869,16 +891,16 @@ gda_mysql_provider_begin_transaction (GdaServerProvider *provider,
 	/* set isolation level */
 	switch (level) {
 	case GDA_TRANSACTION_ISOLATION_READ_COMMITTED :
-		rc = mysql_real_query (mysql, "SET TRANSACTION ISOLATION LEVEL READ COMMITTED", 46);
+		rc = gda_mysql_real_query_wrap (cnc, mysql, "SET TRANSACTION ISOLATION LEVEL READ COMMITTED", 46);
 		break;
 	case GDA_TRANSACTION_ISOLATION_READ_UNCOMMITTED :
-		rc = mysql_real_query (mysql, "SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED", 48);
+		rc = gda_mysql_real_query_wrap (cnc, mysql, "SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED", 48);
 		break;
 	case GDA_TRANSACTION_ISOLATION_REPEATABLE_READ :
-		rc = mysql_real_query (mysql, "SET TRANSACTION ISOLATION LEVEL REPEATABLE READ", 47);
+		rc = gda_mysql_real_query_wrap (cnc, mysql, "SET TRANSACTION ISOLATION LEVEL REPEATABLE READ", 47);
 		break;
 	case GDA_TRANSACTION_ISOLATION_SERIALIZABLE :
-		rc = mysql_real_query (mysql, "SET TRANSACTION ISOLATION LEVEL SERIALIZABLE", 44);
+		rc = gda_mysql_real_query_wrap (cnc, mysql, "SET TRANSACTION ISOLATION LEVEL SERIALIZABLE", 44);
 		break;
 	default :
 		rc = 0;
@@ -890,7 +912,7 @@ gda_mysql_provider_begin_transaction (GdaServerProvider *provider,
 	}
 	else {
 		/* start the transaction */
-		rc = mysql_real_query (mysql, "BEGIN", 5);
+		rc = gda_mysql_real_query_wrap (cnc, mysql, "BEGIN", 5);
 		if (rc != 0) {
 			event = gda_mysql_make_error (mysql);
 			gda_connection_add_event (cnc, event);
@@ -926,7 +948,7 @@ gda_mysql_provider_commit_transaction (GdaServerProvider *provider,
 		return FALSE;
 	}
 
-	rc = mysql_real_query (mysql, "COMMIT", 6);
+	rc = gda_mysql_real_query_wrap (cnc, mysql, "COMMIT", 6);
 	if (rc != 0) {
 		event = gda_mysql_make_error (mysql);
 		gda_connection_add_event (cnc, event);
@@ -961,7 +983,7 @@ gda_mysql_provider_rollback_transaction (GdaServerProvider *provider,
 		return FALSE;
 	}
 
-	rc = mysql_real_query (mysql, "ROLLBACK", 8);
+	rc = gda_mysql_real_query_wrap (cnc, mysql, "ROLLBACK", 8);
 	if (rc != 0) {
 		event = gda_mysql_make_error (mysql);
 		gda_connection_add_event (cnc, event);
@@ -1632,15 +1654,19 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 					   "c.COLUMN_DEFAULT, c.EXTRA, "
 					   "u.REFERENCED_TABLE_NAME, u.REFERENCED_COLUMN_NAME "
 					   "FROM INFORMATION_SCHEMA.COLUMNS c "
-					   "LEFT OUTER JOIN INFORMATION_SCHEMA.KEY_COLUMN_USAGE u "
+					   "LEFT OUTER JOIN "
+					   "(SELECT sub.TABLE_SCHEMA, sub.COLUMN_NAME, sub.TABLE_NAME, "
+					   "sub.REFERENCED_TABLE_NAME, sub.REFERENCED_COLUMN_NAME "
+					   "FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE sub where sub.TABLE_NAME='%s' "
+					   "AND sub.REFERENCED_TABLE_NAME IS NOT NULL) u "
 					   "ON (c.TABLE_NAME = u.TABLE_NAME AND c.TABLE_SCHEMA = u.TABLE_SCHEMA "
 					   "AND c.COLUMN_NAME = u.COLUMN_NAME) "
-					   " WHERE c.TABLE_NAME = '%s' AND c.TABLE_SCHEMA = DATABASE() "
-					   "ORDER BY c.ORDINAL_POSITION;",
-					   table_name);
+					   "WHERE c.TABLE_NAME = '%s' AND c.TABLE_SCHEMA = DATABASE() "
+					   "ORDER BY c.ORDINAL_POSITION",
+					   table_name, table_name);
 	}
 
-	rc = mysql_real_query (mysql, cmd_str, strlen (cmd_str));
+	rc = gda_mysql_real_query_wrap (cnc, mysql, cmd_str, strlen (cmd_str));
 	g_free (cmd_str);
 	if (rc != 0) {
 		gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
@@ -1925,7 +1951,7 @@ get_mysql_constraints (GdaConnection *cnc, GdaParameterList *params)
 	 * Obtaining list of columns 
 	 */
 	cmd_str = g_strdup_printf ("SHOW COLUMNS FROM %s", table_name);
-	rc = mysql_real_query (mysql, cmd_str, strlen (cmd_str));
+	rc = gda_mysql_real_query_wrap (cnc, mysql, cmd_str, strlen (cmd_str));
 	g_free (cmd_str);
 	if (rc != 0) {
 		gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
@@ -2048,7 +2074,7 @@ get_mysql_constraints (GdaConnection *cnc, GdaParameterList *params)
 					   "CONSTRAINT_NAME != 'PRIMARY' AND REFERENCED_TABLE_NAME IS NOT NULL "
 					   "ORDER BY CONSTRAINT_NAME, ORDINAL_POSITION", 
 					   table_name);
-		rc = mysql_real_query (mysql, cmd_str, strlen (cmd_str));
+		rc = gda_mysql_real_query_wrap (cnc, mysql, cmd_str, strlen (cmd_str));
 		g_free (cmd_str);
 		if (rc != 0) {
 			gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
@@ -2144,7 +2170,7 @@ get_mysql_constraints (GdaConnection *cnc, GdaParameterList *params)
 		MYSQL_ROW mysql_row;
 		cmd_str = g_strdup_printf ("SHOW CREATE TABLE %s",
 					   table_name);
-		rc = mysql_real_query (mysql, cmd_str, strlen (cmd_str));
+		rc = gda_mysql_real_query_wrap (cnc, mysql, cmd_str, strlen (cmd_str));
 		g_free (cmd_str);
 		if (rc != 0) {
 			gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
