@@ -1,5 +1,5 @@
 /* GDA MySQL Provider
- * Copyright (C) 1998 - 2005 The GNOME Foundation
+ * Copyright (C) 1998 - 2007 The GNOME Foundation
  *
  * AUTHORS:
  * 	   Michael Lausch <michael@lausch.at>
@@ -24,12 +24,22 @@
 #include <glib/gi18n-lib.h>
 #include <libgda/gda-config.h>
 #include "gda-mysql-provider.h"
+#include <libgda/gda-server-provider-extra.h>
+#include <libgda/binreloc/gda-binreloc.h>
 
+static gchar      *module_path = NULL;
 const gchar       *plugin_get_name (void);
 const gchar       *plugin_get_description (void);
 gchar             *plugin_get_dsn_spec (void);
 GdaServerProvider *plugin_create_provider (void);
 
+void
+plugin_init (const gchar *real_path)
+{
+	if (real_path)
+		module_path = g_strdup (real_path);
+}
+
 const gchar *
 plugin_get_name (void)
 {
@@ -45,18 +55,20 @@ plugin_get_description (void)
 gchar *
 plugin_get_dsn_spec (void)
 {
-	gchar *specs, *file;
-	gsize len;
+	gchar *ret, *dir;
 
-	file = g_build_filename (LIBGDA_DATA_DIR, "mysql_specs_dsn.xml", NULL);
-	if (g_file_get_contents (file, &specs, &len, NULL)) 
-		return specs;
-	else
-		return NULL;
+	dir = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, NULL);
+	ret = gda_server_provider_load_file_contents (module_path, dir, "mysql_specs_dsn.xml");
+	g_free (dir);
+	return ret;
 }
 
 GdaServerProvider *
 plugin_create_provider (void)
 {
-	return gda_mysql_provider_new ();
+	GdaServerProvider *prov;
+
+	prov = gda_mysql_provider_new ();
+	g_object_set_data (prov, "GDA_PROVIDER_DIR", module_path);
+	return prov;
 }
