@@ -27,7 +27,6 @@
 #include <string.h>
 #include <libgda/gda-data-model-array.h>
 #include <libgda/gda-server-provider-extra.h>
-#include <libgda/gda-parameter-list.h>
 #include "gda-odbc.h"
 #include "gda-odbc-provider.h"
 
@@ -108,6 +107,7 @@ gda_odbc_provider_class_init (GdaOdbcProviderClass *klass)
 	provider_class->string_to_value = NULL;
 	provider_class->get_def_dbms_type = NULL;
 
+	provider_class->create_connection = NULL;
 	provider_class->open_connection = gda_odbc_provider_open_connection;
 	provider_class->close_connection = gda_odbc_provider_close_connection;
 	provider_class->get_database = gda_odbc_provider_get_database;
@@ -151,7 +151,7 @@ gda_odbc_provider_get_type (void)
 {
         static GType type = 0;
 
-        if (!type) {
+        if (G_UNLIKELY (type == 0)) {
                 static GTypeInfo info = {
                         sizeof (GdaOdbcProviderClass),
                         (GBaseInitFunc) NULL,
@@ -302,6 +302,7 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql, GdaC
 	SQLRETURN rc;
 	SQLCHAR *sql_as_ansi;
 	int i;
+	GdaConnectionEvent *event;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
@@ -309,6 +310,10 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql, GdaC
         if (!priv_data)
                 return NULL;
 
+	event = gda_connection_event_new (GDA_CONNECTION_EVENT_COMMAND);
+	gda_connection_event_set_description (event, sql);
+	gda_connection_add_event (cnc, event);
+
 	sql_as_ansi = (SQLCHAR *) g_locale_from_utf8 (sql, -1, NULL, NULL, NULL);
 	rc = SQLExecDirect (priv_data->hstmt, sql_as_ansi, SQL_NTS);
 	g_free (sql_as_ansi);
