@@ -58,6 +58,7 @@ typedef struct {
 } GdaPostgresTypeOid;
 
 typedef struct {
+	GdaConnection      *cnc;
 	PGconn             *pconn;
 	gint                ntypes;
 	GdaPostgresTypeOid *type_data;
@@ -74,6 +75,10 @@ typedef struct {
 
 	/* OID of the last inserted tuple */
 	Oid                 last_insert_id;
+
+	/* Cursor usage */
+	gboolean            use_cursor;
+	gint                chunck_size;
 } GdaPostgresConnectionData;
 
 /* NOTE ABOUT THE POSTGRES VERSIONS:
@@ -90,7 +95,7 @@ typedef struct {
 
 G_BEGIN_DECLS
 
-GType              gda_postgres_provider_get_type (void);
+GType              gda_postgres_provider_get_type (void) G_GNUC_CONST;
 GdaServerProvider *gda_postgres_provider_new (void);
 
 G_END_DECLS
